## NOTES:
## Wind force in 1853-1859 not formatted (don't know how to convert lbs/ft2 to wind speed)
## Temperature in the Sun and on Grass not formatted
## Hourly Tmax/Tmin for 23-24/12/1854 not formatted
## Data for 1860 (HMS Meander) were not formatted (quality seems very poor)
## Ozone amount (1863-1865) not formatted

library(dataresqc)
library(XLConnect)
library(plyr)

## Create output directory (if necessary)
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}


## 1853-1859

## SUBDAILY
station_code <- paste(name_group, name_folder, 1, sep = "_")
files <- c("Africa_T3_GB_AscensionIsland_1853-1854_subdaily.xls",
           "Africa_T3_GB_AscensionIsland_1855-1856_subdaily.xlsx",
           "Africa_T3_GB_AscensionIsland_1857-1859_subdaily.xls")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily"), ]

## Loop on files
out <- list()
for (i in 1:length(files)) {
  
  ## Read data
  template <- readWorksheetFromFile(paste0("../../3_Revised/AscensionIsland/",files[i]),
                                    sheet=1, startRow=9, header=FALSE, colTypes="character")
  
  ## Define j index
  if (i == 1) {
    j <- 22
  } else {
    j <- 20
  }
  
  ## Time
  times <- c(template[,c(4,j)], recursive=TRUE)
  hours <- as.integer(substr(times,12,13))
  minutes <- as.integer(substr(times,15,16))
  meta_time <- paste0("orig.time=", c(substr(template[,4],12,16),substr(template[,j],12,16)))
  
  ## Initialize data frame
  out[[i]] <- data.frame(year = rep(as.integer(template[,1]), 2), 
                         month = rep(as.integer(template[,2]), 2), 
                         day = rep(as.integer(template[,3]), 2), 
                         hour = hours, 
                         minute = minutes)
  
  ## Pressure
  pres <- c(template[,c(ifelse(i==1,6,5),j+1)], recursive=TRUE)
  atb <- c(template[,c(ifelse(i==1,7,6),j+2)], recursive=TRUE)
  out[[i]]$p_meta <- paste0(meta_time, "|orig=", pres, "in|atb=", atb, "F")
  if (i == 1) {
    k <- which(!is.na(template[,5]))
    pres_times <- substr(times, 12, 16)
    pres_times[k] <- substr(template[k,5], 12, 16)
    out[[i]]$p_meta[k] <- paste0("orig.time=", pres_times[k], 
                                 "|orig=", pres[k], "in|atb=", atb[k], "F")
    pres_times <- pres_times[order(out[[i]]$year,out[[i]]$month,out[[i]]$day,out[[i]]$hour)]
  }
  h <- which(is.na(as.numeric(atb)))
  atb[h] <- 0
  out[[i]]$p <- round(convert_pressure(as.numeric(pres), 25.4, 
                                       as.numeric(metadata$Latitude), 
                                       as.numeric(metadata$Altitude),
                                       (as.numeric(atb)-32)*5/9), 
                      1)
  out[[i]]$p_meta[h] <- paste0(out[[i]]$p_meta[h], "|PTC=N")

  ## Temperature
  if (i == 1) {
    temp <- c(template[,c(9,j+4)], recursive=TRUE)
    out[[i]]$ta <- round((as.numeric(temp)-32)*5/9, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", temp, "F")
  }
  
  ## Wind direction
  directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                  "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                  "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                  "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")
  wdir <- c(template[,c(j-12,j+ifelse(i==1,5,4))], recursive=TRUE)
  out[[i]]$dd <- round(11.25*(match(wdir,directions)-1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", wdir)
  
  ## Tmax
  temp <- c(template[,j-8], rep(NA,nrow(template)))
  out[[i]]$Tx <- round((as.numeric(temp)-32)*5/9, 1)
  out[[i]]$Tx_meta <- paste0(meta_time, "|orig=", temp, "F")
  
  ## Tmin
  temp <- c(template[,j-7], rep(NA,nrow(template)))
  out[[i]]$Tn <- round((as.numeric(temp)-32)*5/9, 1)
  out[[i]]$Tn_meta <- paste0(meta_time, "|orig=", temp, "F")
  
  ## Tb_max
  temp <- c(template[,j-5], rep(NA,nrow(template)))
  out[[i]]$tb_max <- round((as.numeric(temp)-32)*5/9, 1)
  out[[i]]$tb_max_meta <- paste0(meta_time, "|orig=", temp, "F")
  
  ## Tb_min
  temp <- c(template[,j-4], rep(NA,nrow(template)))
  out[[i]]$tb_min <- round((as.numeric(temp)-32)*5/9, 1)
  out[[i]]$tb_min_meta <- paste0(meta_time, "|orig=", temp, "F")

  ## Precipitation on ground
  rain <- c(template[,j-2], rep(NA,nrow(template)))
  out[[i]]$rr1_meta <- paste0(meta_time, "|orig=", rain, "in")
  rain[which(rain%in%c(NA,"-"))] <- 0
  rain[which(rain%in%c("NA","R"))] <- NA
  rain[which(hours==15)] <- NA
  out[[i]]$rr1 <- round(as.numeric(rain) * 25.4, 1)
  
  ## Precipitation at 22 feet
  rain <- c(template[,j-1], rep(NA,nrow(template)))
  out[[i]]$rr2_meta <- paste0(meta_time, "|orig=", rain, "in")
  rain[which(rain%in%c(NA,"-"))] <- 0
  rain[which(rain%in%c("NA","R"))] <- NA
  rain[which(hours==15)] <- NA
  out[[i]]$rr2 <- round(as.numeric(rain) * 25.4, 1)
  
}

## Merge data frames
out <- Reduce(rbind.fill, out)
out <- out[order(out$year,out$month,out$day,out$hour),]
out$p_hour <- out$hour
out$p_minute <- out$minute
out$p_hour[1:length(pres_times)] <- as.integer(substr(pres_times,1,2))
out$p_minute[1:length(pres_times)] <- as.integer(substr(pres_times,4,5))


## Additional hourly data for 23-24/12/1854
template <- readWorksheetFromFile("../../3_Revised/AscensionIsland/Africa_T3_GB_AscensionIsland_1854_hourly.xls",
                                  sheet=1, startRow=9, header=FALSE, colTypes="character")

## Time
times <- template[,4]
hours <- as.integer(substr(times,12,13))
minutes <- as.integer(substr(times,15,16))
meta_time <- paste0("orig.time=", substr(template[,4],12,16))

## Initialize data frame
out_add <- data.frame(year = as.integer(template[,1]), 
                      month = as.integer(template[,2]), 
                      day = as.integer(template[,3]), 
                      hour = hours, 
                      minute = minutes)

## Pressure
pres <- template[,5]
atb <- template[,6]
out_add$p_meta <- paste0(meta_time, "|orig=", pres, "in|atb=", atb, "F")
out_add$p <- round(convert_pressure(as.numeric(pres), 25.4, 
                                    as.numeric(metadata$Latitude), 
                                    as.numeric(metadata$Altitude),
                                    (as.numeric(atb)-32)*5/9), 
                   1)
out_add$p_hour <- out_add$hour
out_add$p_minute <- out_add$minute

## Wind direction
wdir <- template[,7]
out_add$dd <- round(11.25*(match(wdir,directions)-1), 0)
out_add$dd_meta <- paste0(meta_time, "|orig=", wdir)

## Merge data frames
out <- rbind.fill(out, out_add)
out <- out[order(out$year,out$month,out$day,out$hour),]


## Write SEF files
write_sef(out[,c("year","month","day","p_hour","p_minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=Y"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","Tx")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "Tx",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$Tx_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","Tn")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "Tn",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$Tn_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","tb_max")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "tb",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tb_max_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE,
          note = "max")

write_sef(out[,c("year","month","day","hour","minute","tb_min")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "tb",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$tb_min_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE,
          note = "min")

write_sef(out[,c("year","month","day","hour","minute","rr1")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer, " | Height=ground"),
          meta = out$rr1_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE,
          note = "ground")

write_sef(out[,c("year","month","day","hour","minute","rr2")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer, " | Height=22 feet"),
          meta = out$rr2_meta,
          period = "day",
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = FALSE,
          note = "22ft")


## 1863-1865

## DAILY
## Read metadata
station_code <- paste(name_group, name_folder, 3, sep = "_")
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "daily"), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/AscensionIsland/Africa_T3_GB_AscensionIsland_1863-1865_daily.xls",
                                  sheet=1, startRow=9, header=FALSE, colTypes="character")

## Initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]), 
                  hour = rep(NA, nrow(template)), 
                  minute = rep(NA, nrow(template)))

## Precipitation
out$rr <- round(as.numeric(template[,4]) * 25.4, 1)
out$rr_meta <- paste0("orig=", template[,4], "in")

## Wind speed
out$w_max <- round(as.numeric(template[,5]) * 1609.34 / 3600, 1)
out$w_max_meta <- paste0("orig=", template[,5], "mph")
out$w_min <- round(as.numeric(template[,6]) * 1609.34 / 3600, 1)
out$w_min_meta <- paste0("orig=", template[,6], "mph")
out$w_mean <- round(as.numeric(template[,7]) * 1609.34 / (24*3600), 1)
out$w_mean_meta <- paste0("orig=", template[,7], "miles/24h")

## Wind direction
wdir <- template[,8]
wdir <- gsub(" ", "", wdir)
wdir <- gsub("by", "b", wdir)
out$dd <- round(11.25*(match(wdir,directions)-1), 0)
out$dd[which(wdir=="SE1/2S")] <- 141
out$dd[which(wdir=="SE1/2E")] <- 129
out$dd[which(wdir=="SE1/4S")] <- 138
out$dd[which(wdir=="SE1/4E")] <- 132
out$dd[which(wdir=="SE3/4S")] <- 143
out$dd[which(wdir=="SEbE1/4E")] <- 121
out$dd[which(wdir=="EbS1/2S")] <- 107
out$dd_meta <- paste0("orig=", template[,8])

## SST
out$t_water <- round((as.numeric(template[,9])-32)*5/9, 1)
out$t_water_meta <- paste0("orig=", template[,9], "F")

## Write SEF files
write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "day",
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w_max")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "m/s",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_max_meta,
          period = "day",
          keep_na = TRUE,
          note = "max")

write_sef(out[,c("year","month","day","hour","minute","w_min")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "m/s",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_min_meta,
          period = "day",
          keep_na = TRUE,
          note = "min")

write_sef(out[,c("year","month","day","hour","minute","w_mean")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "m/s",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_mean_meta,
          period = "day",
          keep_na = TRUE,
          note = "mean")

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = "day",
          keep_na = TRUE,
          note = "mean")

write_sef(out[,c("year","month","day","hour","minute","t_water")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "t_water",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$t_water_meta,
          period = "day",
          keep_na = TRUE,
          note = "mean")


## HOURLY
## Read metadata
station_code <- paste(name_group, name_folder, 3, sep = "_")
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "hourly"), ]

## Time
n <- diff(as.Date(c("1863-09-01","1865-11-01")))
hours <- rep(1:24, n)
minutes <- rep(0, 24*n)
meta_time <- paste0("orig.time=", hours)

## Pressure
template <- readWorksheetFromFile("../../3_Revised/AscensionIsland/Africa_T3_GB_AscensionIsland_1863-1865_hourly-pressure.xls",
                                  sheet=1, startRow=9, header=FALSE, colTypes="character")
out <- data.frame(year = rep(as.integer(template[,1]), each=24), 
                  month = rep(as.integer(template[,2]), each=24), 
                  day = rep(as.integer(template[,3]), each=24), 
                  hour = hours, 
                  minute = minutes)
pres <- c(t(template[,4:27]), recursive=TRUE)
out$p <- round(convert_pressure(as.numeric(pres), 25.4, as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude)), 1)
out$p_meta <- paste0(meta_time, "|orig=", pres, "in")
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "mslp",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=Y | ",
                            "Instrument=Standard Barometer nr.556 corrected for index error"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

## Temperature
template <- readWorksheetFromFile("../../3_Revised/AscensionIsland/Africa_T3_GB_AscensionIsland_1863-1865_hourly-temp.xlsx",
                                  sheet=1, startRow=9, header=FALSE, colTypes="character")
temp <- c(t(template[,4:27]), recursive=TRUE)
out$ta <- round((as.numeric(temp)-32) * 5/9, 1)
out$ta_meta <- paste0(meta_time, "|orig=", temp, "F")
write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | Instrument=Dry Bulb Thermometer nr.81 corrected for scale error"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

## Wet bulb
template <- readWorksheetFromFile("../../3_Revised/AscensionIsland/Africa_T3_GB_AscensionIsland_1863-1865_hourly-wetbulb.xls",
                                  sheet=1, startRow=9, header=FALSE, colTypes="character")
wetbulb <- c(t(template[,4:27]), recursive=TRUE)
out$tb <- round((as.numeric(wetbulb)-32) * 5/9, 1)
out$tb_meta <- paste0(meta_time, "|orig=", wetbulb, "F")
write_sef(out[,c("year","month","day","hour","minute","tb")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "tb",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | Instrument=Wet Bulb Thermometer nr.82 corrected for scale error"),
          meta = out$tb_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)

## Cloudiness
template <- readWorksheetFromFile("../../3_Revised/AscensionIsland/Africa_T3_GB_AscensionIsland_1863-1865_hourly-cloudiness.xlsx",
                                  sheet=1, startRow=9, header=FALSE, colTypes="character")
template <- template[, -seq(5,51,2)]
clouds <- c(t(template[,4:27]), recursive=TRUE)
out$n <- round(as.numeric(clouds)*10, 0)
out$n_meta <- paste0(meta_time, "|orig=", clouds)
write_sef(out[,c("year","month","day","hour","minute","n")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "n",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "%",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$n_meta,
          period = 0,
          time_offset = as.numeric(metadata$Longitude) * 12 / 180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
