## NOTE: Temperature was not formatted because of a clear time shift until 1836 (also it is already available in Dove)

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Read data and metadata
template <- readWorksheetFromFile("../../3_Revised/RioDeJaneiro/Americas_T3_BR_RioDeJaneiro_1833-1841_monthly.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code), ]
template[,1] <- fill_variable(template[,1], NA)

## Initialize data frame
out <- data.frame(year=as.integer(template[,1]), 
                  month=as.integer(template[,2]), 
                  day=rep("", nrow(template)), 
                  hour=rep("", nrow(template)), 
                  minute=rep("", nrow(template)))

## Temperature (Fahrenheit)
#out$ta <- round((as.numeric(template[,5]) - 32) * 5 / 9, 1)
#out$ta_meta <- paste0("orig=", template[,5], "F")

## Wet days
out$nr <- as.integer(template[,8])

## Write SEF files
#write_sef(out[, c("year","month","day","hour","minute","ta")],
#          outpath = paste0("../../4_Formatted/", name_folder),
#          variable = "ta",
#          cod = metadata$Code,
#          nam = metadata$Station,
#          lat = metadata$Latitude,
#          lon = metadata$Longitude,
#          alt = metadata$Altitude,
#          sou = "PALAEO-RA",
#          link = metadata$C3S.Link,
#          units = "C",
#          stat = "mean",
#          metaHead = paste0("Observer=", metadata$Observer, " | Obs.times=noon"),
#          meta = out$ta_meta,
#          period = "month",
#          note = "monthly",
#          keep_na = TRUE)
          
write_sef(out[, c("year","month","day","hour","minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "days",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
