##

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Read data and metadata
template <- readWorksheetFromFile("../../3_Revised/Savanna-la-Mar/Americas_T3_JM_Savanna-la-Mar_1752-1786_monthly-wetdays.xls",
                                  sheet=1, startRow=8, endRow=42, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code), ]

## Initialize data frame
out <- data.frame(year=rep(as.integer(template[,1]), each=12), 
                  month=rep(1:12, nrow(template)), 
                  day=rep("", nrow(template)*12), 
                  hour=rep("", nrow(template)*12), 
                  minute=rep("", nrow(template)*12))

## Wet days
out$nr <- as.integer(c(t(template[,2:13]),recursive=TRUE))

## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "days",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)