##

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Read data and metadata
template <- readWorksheetFromFile("../../3_Revised/Veracruz/Americas_T3_MX_Veracruz_1791-1804_monthly-temp.xls",
                                  sheet=1, startRow=6, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code), ]

## Initialize data frame
out <- data.frame(year=rep(as.integer(template[1,-1]),each=12), 
                  month=rep(1:12, ncol(template)-1), 
                  day=rep("", (ncol(template)-1)*12), 
                  hour=rep("", (ncol(template)-1)*12), 
                  minute=rep("", (ncol(template)-1)*12))

## Temperature
out$ta <- as.numeric(c(template[2:13,-1],recursive=TRUE))

## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          note = "monthly",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
