## NOTES:
## The record was split into two parts because they are separated by a 30-year gap
## and because only the first part has monthly means.
## Temperature was reduced to Deluc's scale by Cotte.

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## Read data
template <- readWorksheetFromFile("../../3_Revised/Pondicherry/Asia_T3_IN_Pondicherry_1736-1769_monthly-temp.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template[,1] <- fill_variable(template[,1], NA)

## Arrange output
out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = rep("", nrow(template)),
                  Hour = rep("", nrow(template)),
                  Minute = rep("", nrow(template)),
                  Mean = round(1.25 * as.numeric(template[,5]), 1),
                  MR = round(1.25 * (as.numeric(template[,3]) +
                    as.numeric(template[,4])) / 2, 1),
                  Meta = paste0("orig=", template[,5], "R"),
                  Meta_MR = paste0("orig=", template[,3], "+", template[,4], "R"),
                  stringsAsFactors = FALSE)


## PART 1
metadata <- inventory[which(inventory$Code == paste0(station_code,"_1")), ]

## Write SEF files
write_sef(out[which(out$Year<=1739), 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          meta = out$Meta[which(out$Year<=1739)],
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          note = "monthly",
          keep_na = TRUE)


## PART 2
metadata <- inventory[which(inventory$Code == paste0(station_code,"_2")), ]

## Write SEF files
write_sef(out[which(out$Year>1739), c(1:5,7)],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mid_range",
          meta = out$Meta_MR[which(out$Year>1739)],
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          note = "monthly",
          keep_na = TRUE)


file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub(" ", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)