#rm(list=ls())

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "monthly"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)


############################
## Monthly
############################
template1 <- readWorksheetFromFile("../../3_Revised/Burnley/Europe_T3_GB_Burnley_Townley_1677-1693_monthly-rain.xlsx",
                                  sheet=1, startRow=6, header=T, colTypes="character")
template2 <- readWorksheetFromFile("../../3_Revised/Burnley/Europe_T3_GB_Burnley_Townley_1697-1698_monthly-rain.xlsx",
                                   sheet=1, startRow=6, header=T, colTypes="character")
template3 <- readWorksheetFromFile("../../3_Revised/Burnley/Europe_T3_GB_Burnley_Townley_1703-1704_monthly-rain.xlsx",
                                   sheet=1, startRow=6, header=T, colTypes="character")

## 1/100 of inch to inch
template1[,3] <- as.numeric(template1[,3])
template1[,3] <- template1[,3]/100

## 1/5 of inch to inch
template2[,3] <- as.numeric(template2[,3])
template2[,3] <- template2[,3]/5
template3[,3] <- as.numeric(template3[,3])
template3[,3] <- template3[,3]/5
## all templates together
names(template1) <- names(template2) <- names(template3)
template <- rbind(template1, template2, template3)

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    Tx = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    Tn = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    p  = function(x) {return(round(convert_pressure(as.numeric(x),f = 25.4,lat = lat,alt = alt), 1))},
                    rr = function(x) {round(as.numeric(x)*25.4,1)})

### time ###
time_offset <- as.numeric(lon)*12/180

Year <- as.integer(fill_val(template[,"Year"]))
Month <- as.integer(template[,"Month"])

############## 
#### Precipitation
##############
message("rr")
dat <- template[,3]
vardat <- conversions[["rr"]](dat)

out <- data.frame(Year = Year,
                  Month = Month,
                  Day = rep(NA, length(Year)),
                  Hour = rep(NA, length(Year)),
                  Minute = rep(NA, length(Year)),
                  Prec = vardat,
                  stringsAsFactors = FALSE)

template1 <- readWorksheetFromFile("../../3_Revised/Burnley/Europe_T3_GB_Burnley_Townley_1677-1693_monthly-rain.xlsx",
                                   sheet=1, startRow=6, header=T, colTypes="character")
template2 <- readWorksheetFromFile("../../3_Revised/Burnley/Europe_T3_GB_Burnley_Townley_1697-1698_monthly-rain.xlsx",
                                   sheet=1, startRow=6, header=T, colTypes="character")
template3 <- readWorksheetFromFile("../../3_Revised/Burnley/Europe_T3_GB_Burnley_Townley_1703-1704_monthly-rain.xlsx",
                                   sheet=1, startRow=6, header=T, colTypes="character")
names(template1) <- names(template2) <- names(template3)
template <- rbind(template1, template2, template3)

meta_col <- paste0("orig=",template[1:180,3],"(1/100 of inch)")
meta_col <- append(meta_col, paste0("orig=",template[181:219,3],"(1/5 of inch)"))

## Write SEF files
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian"),
          meta = meta_col,
          period = "month",
          time_offset = time_offset,
          keep_na = TRUE,
          outfile=paste0("PALAEO-RA_",metadata$Code,"_",metadata$First.Year,"-",metadata$Last.Year,"_","rr","_","monthly",".tsv"))

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
