## NOTES:

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## Read temmplate
template <- readWorksheetFromFile("../../3_Revised/Canterbury/Europe_T3_GB_Canterbury_1734_monthly-prec.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template[,1] <- fill_variable(template[,1], NA)

metadata <- inventory[which(inventory$Code == station_code), ]

## Write output
out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = rep("", nrow(template)),
                  Hour = rep("", nrow(template)),
                  Minute = rep("", nrow(template)),
                  Value = round(as.numeric(template[,3])*25.4, 1),
                  Meta = paste0("orig=", template[,3], "in"),
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub(" ", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)