##

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
inventory$Latitude <- as.numeric(inventory$Latitude)
inventory$Longitude <- as.numeric(inventory$Longitude)

## Read metadata from data file
metadata <- readWorksheetFromFile("../../3_Revised/Giornale-AstroMeteorologico/Europe_T3_IT_Giornale-AstroMeteorologico_1778-1810_monthly-prec.xlsx",
                                  sheet=2, startRow=1, header=TRUE, colTypes="character")

## Read template
template <- readWorksheetFromFile("../../3_Revised/Giornale-AstroMeteorologico/Europe_T3_IT_Giornale-AstroMeteorologico_1778-1810_monthly-prec.xlsx",
                                  sheet=1, startRow=6, header=FALSE, colTypes="character")
stations <- template[1, ]
template <- template[-c(1,2), ]
template <- template[which(!is.na(template[,2])), ]

## Loop on stations
for (i in 1:nrow(metadata)) {
  
  j <- which(stations == metadata$Station[i])
  lat <- round(as.numeric(metadata$Latitude[i]), 2)
  lon <- round(as.numeric(metadata$Longitude[i]), 2)
  stid <- inventory$Code[which(round(inventory$Latitude,2) == lat &
                                 round(inventory$Longitude,2) == lon &
                                 inventory$First.Year == metadata$First.Year[i] &
                                 inventory$Last.Year == metadata$Last.Year[i])]
  stnam <- inventory$Station[which(round(inventory$Latitude,2) == lat &
                                     round(inventory$Longitude,2) == lon &
                                     inventory$First.Year == metadata$First.Year[i] &
                                     inventory$Last.Year == metadata$Last.Year[i])]
  prec <- as.numeric(template[,j]) + as.numeric(template[,j+1])/12
  meta_col <- paste0("orig=", template[,j], "Pin ", template[,j+1], "Pl")
  
  if (j == 5) { # Chioggia
    prec[1:12] <- prec[1:12] + as.numeric(template[1:12,j+2])/144
    meta_col[1:12] <- paste0(meta_col[1:12], " ", template[1:12,j+2], "Pl/12")
  }
  if (j == 20) { # Padova
    prec[26:35] <- prec[26:35] * 25.4 / 27.07
    meta_col[26:35] <- paste0("orig=", template[26:35,j], "in")
  }
  
  out <- data.frame(Year = as.integer(template[,1]),
                    Month = as.integer(template[,2]),
                    Day = rep("", nrow(template)),
                    Hour = rep("", nrow(template)),
                    Minute = rep("", nrow(template)),
                    Value = round(prec*27.07, 1),
                    stringsAsFactors = FALSE)
  
  write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = "rr",
            cod = stid,
            nam = stnam,
            lat = metadata$Latitude[i],
            lon = metadata$Longitude[i],
            alt = NA,
            sou = "PALAEO-RA",
            link = NA,
            units = "mm",
            stat = "sum",
            meta = meta_col,
            metaHead = paste0("Observer=", metadata$Observer[i]),
            period = "month",
            note = "monthly",
            keep_na = FALSE)
  
}


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)