## NOTES:

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## Read data
template <- readWorksheetFromFile("../../3_Revised/Harnosand/Europe_T3_SE_Harnosand_1728_monthly-pres.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code), ]
template[,1] <- fill_variable(template[,1], NA)
for (j in 4:9) { # add leading zeros
  k <- which(nchar(template[,j])==1)
  template[k,j] <- paste0("0", template[k,j])
}

## Mid-range pressure
pres <- (as.numeric(template[,3]) + as.numeric(template[,4])/100 +
           as.numeric(template[,7]) + as.numeric(template[,8])/100) / 2
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))

out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = rep("", nrow(template)),
                  Hour = rep("", nrow(template)),
                  Minute = rep("", nrow(template)),
                  Value = round(pres, 1),
                  Meta = paste0("orig=", template[,3], ".", template[,4], "+", 
                                template[,7], ".", template[,8], "in"),
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub(" ", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
