## NOTES:
## For the barometer I assumed that the Rheinländer Zoll is used, as it best fits the data, but
## there is no metadata supporting that.
## Thermometer and hygrometer were not formatted.

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)

template <- readWorksheetFromFile("../../3_Revised/Kaliningrad/Europe_T3_RU_Kaliningrad_1702_subdaily.xls",
                                  sheet=1, startRow=7, header=F, colTypes="character")

conversions <- list(p  = function(x) round(convert_pressure(as.numeric(x),f = 26.154,lat = lat,alt = alt), 1))

### Time ###
Year <- as.integer(fill_val(template[,1]))
Month <- as.integer(fill_val(template[,2]))
Day <- as.integer(fill_val(template[,3]))
hours <- rep(NA, length(Year))
minutes <- hours
meta_time <- paste0("orig.time=", template[,4])


## Pressure
pres <- as.numeric(template[,6]) + as.numeric(template[,7])/12
pres <- conversions[["p"]](pres)
out <- data.frame(Year,Month,Day,hours,minutes,pres)
meta_col <- paste0(meta_time,"|orig=",template[,6],".",template[,7],"Rh.in")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|PTC=N|PGC=Y"),
          meta = meta_col,
          period = 0,
          keep_na = TRUE)


## Wind direction
directions <- c("N", "NtO", "NNO", "NOtN", "NO", "NOtO", "ONO", "OtN", "O", "OtS", "OSO", "SOtO",
                "SO", "SOtS", "SSO", "StO", "S", "StW", "SSW", "SWtS", "SW", "SWtW", "WSW", "WtS",
                "W", "WtN", "WNW", "NWtW", "NW", "NWtN", "NNW", "NtW")
wdir <- round(11.25 * (match(template[,5], directions) - 1), 0)
out <- data.frame(Year,Month,Day,hours,minutes,wdir)
meta_col <- paste0(meta_time,"|orig=",template[,5])
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
