## The years 1621-1622 are actually from Kepler from various places (see Lenke, 1960),
## so they have not been formatted.
## In 1641 the observer moved to Rotenburg (ca. 30 km from Kassel), so the series was splitted into two.

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == paste0(station_code,"_1") & inventory$Resolution == "monthly"), ]

template <- readWorksheetFromFile("../../3_Revised/Kassel/Europe_T3_DE_Kassel_1621-1650_monthly.xls",
                                  sheet=1, startRow=32, header=F, colTypes="character")


### Kassel ###
Year <- as.integer(fill_val(template[1:(18*12),1]))
Month <- as.integer(template[1:(18*12),2])
Day <- rep(NA,length(Month))
hours <- rep(NA,length(Month))
minutes <-rep(NA,length(Month))

###### Rainy days ########

dat <- as.numeric(template[1:(18*12),10])
meta_col <- paste0("orig=",template[1:(18*12),10])
out <- data.frame(Year,Month,Day,hours,minutes,dat)

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "day",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = "month",
          keep_na = TRUE,
          outfile=paste0("PALAEO-RA_",metadata$Code,"_",metadata$First.Year,"-",metadata$Last.Year,"_nr.tsv"))


### Rotenburg ###
metadata <- inventory[which(inventory$Code == paste0(station_code,"_2") & inventory$Resolution == "monthly"), ]
Year <- as.integer(fill_val(template[(18*12+1):(28*12),1]))
Month <- as.integer(template[(18*12+1):(28*12),2])
Day <- rep(NA,length(Month))
hours <- rep(NA,length(Month))
minutes <-rep(NA,length(Month))

###### Rainy days ########
dat <- as.numeric(template[(18*12+1):(28*12),10])
meta_col <- paste0("orig=",template[(18*12+1):(28*12),10])
out <- data.frame(Year,Month,Day,hours,minutes,dat)

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "day",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = "month",
          keep_na = TRUE,
          outfile=paste0("PALAEO-RA_",metadata$Code,"_",metadata$First.Year,"-",metadata$Last.Year,"_nr.tsv"))

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
