## 

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, 1, sep = "_")
 
## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)

template <- readWorksheetFromFile("../../3_Revised/Kiel/Europe_T3_DE_Kiel_1767-1768_subdaily.xlsx",
                                  sheet=1, startRow=6, header=T, colTypes="character")

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    p  = function(x) round(convert_pressure(as.numeric(x),f = 27.07,lat = lat,alt = alt), 1),
                    dd = function(x) round(11.25 * (match(x, directions) - 1), 0))

### time ###
time_offset <- lon*12/180
Year <- as.integer(fill_val(template[,"Year"]))
Month <- as.integer(fill_val(template[,"Month"]))
Day <- as.integer(fill_val(template[,"Day"]))
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- as.integer(template[,"Hora"])
minutes <- as.integer(rep(0,length(Year)))
meta_time <- paste0("orig.time=", template[,"Hora"])

## add 12 hours to PM observations (assuming first observation is always AM)
lastdate <- dates[1]
n <- sum(dates==lastdate)
for (i in 2:length(dates)) {
  if (dates[i] == lastdate) {
    if (n == 2) {
      hours[i] <- hours[i] + 12
    } else if (n > 2) {
      if (hours[i] <= hours[i-1]) {
        hours[i] <- hours[i] + 12
      }
    }
  } else {
    lastdate <- dates[i]
    n <- sum(dates==lastdate)
  }
}


## Pressure
template[,5] <- fill_val(template[,5])
pres <- as.numeric(template[,5]) + as.numeric(template[,6])/100
pres <- conversions[["p"]](pres)
out <- data.frame(Year,Month,Day,hours,minutes,pres)
j <- which(nchar(template[,6])==1)
template[j,6] <- paste0("0", template[j,6])
meta_col <- paste0(meta_time,"|orig=",template[,5],".",template[,6],"Pin")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|PTC=N|PGC=Y"),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


## Temperature
temp <- conversions[["ta"]](as.numeric(template[,7]))
out <- data.frame(Year,Month,Day,hours,minutes,temp)
meta_col <- paste0(meta_time,"|orig=",template[,7],"F")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


## Wind direction
directions <- c("N", "NgO", "NNO", "NOgN", "NO", "NOgO", "ONO", "OgN", "O", "OgS", "OSO", "SOgO",
                "SO", "SOgS", "SSO", "SgO", "S", "SgW", "SSW", "SWgS", "SW", "SWgW", "WSW", "WgS",
                "W", "WgN", "WNW", "NWgW", "NW", "NWgN", "NNW", "NgW")
wdir <- conversions[["dd"]](template[,8])
out <- data.frame(Year,Month,Day,hours,minutes,wdir)
meta_col <- paste0(meta_time,"|orig=",template[,8])
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
