## NOTE: Wet days formatted only until 1850, afterwards they often exceed 30

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "monthly"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
# Altitude not available
alt <- as.numeric(metadata$Altitude)

############################
## Monthly
############################
template <- readWorksheetFromFile("../../3_Revised/Lille/Europe_T3_FR_Lille_1757-1888_monthly.xlsx",
                                  sheet=1, startRow=6, header=T, colTypes="character")

conversions <- list(p  = function(x) round(convert_pressure(as.numeric(x),f = 1,lat = lat,alt = alt), 1))

### time ###
Year <- as.integer(fill_val(template[,"Year"]))
Month <- as.integer(template[,"Month"])
Day <- rep(NA,length(Month))
hours <- rep(NA,length(Month))
minutes <-rep(NA,length(Month))

###### Rainy day ########
dat <- as.numeric(template[,5]) + as.numeric(template[,6])
meta_col <- paste0("orig=",template[,5]," wet days + ",template[,6],"snow days")
vardat <- dat
out <- data.frame(Year,Month,Day,hours,minutes,vardat)

write_sef(out[which(out$Year<=1850),],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "day",
          stat = "sum",
          metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer))),
          meta = meta_col[which(out$Year<=1850)],
          period = "month",
          time_offset = 0,
          keep_na = TRUE,
          outfile=paste0("PALAEO-RA_",metadata$Code,"_",metadata$First.Year,"-",1850,"_nr.tsv"))


###### MINIMUM PRESSURE ########
dat <- as.numeric(template[,4])
vardat <- conversions[["p"]](dat)
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
meta_col <- paste0("orig=",template[,4],"mmHg")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)), " | PGC=Y | PTC=N"),
          meta = meta_col,
          period = "month",
          time_offset = 0,
          keep_na = TRUE,
          outfile=paste0("PALAEO-RA_",metadata$Code,"_",metadata$First.Year,"-",metadata$Last.Year,"_p_monthly-minimum.tsv"))

###### MAXIMUM PRESSURE ########
dat <- as.numeric(template[,3])
vardat <- conversions[["p"]](dat)
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
meta_col <- paste0("orig=",template[,3],"mmHg")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)), " | PGC=Y | PTC=N"),
          meta = meta_col,
          period = "month",
          time_offset = 0,
          keep_na = TRUE,
          outfile=paste0("PALAEO-RA_",metadata$Code,"_",metadata$First.Year,"-",metadata$Last.Year,"_p_monthly-maximum.tsv"))



## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
