## NOTE London_1 (Locke) was not formatted (unknown temperature scale)

#setwd("/scratch3/PALAEO-RA/DataRescue/Projects/Europe/Scripts/London")

#rm(list=ls())

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## GRESHAM COLLEGE ##

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste0(paste(name_group, name_folder, sep = "_"),"_2")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "weekly"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
# Altitude not available
alt <- as.numeric(metadata$Altitude)

############################
## Weekly
############################
template <- readWorksheetFromFile("../../3_Revised/London/Europe_T3_GB_London_Gresham_1695-1696_weekly-rain.xlsx",
                                  sheet=1, startRow=6, header=T, colTypes="character")

conversions <- list(rr = function(x) round(as.numeric(x)*25.4,1))

### time ###
Year <- as.integer(fill_val(template[,"Year"]))
Month <- as.integer(fill_val(template[,"Month"]))
Day <- as.integer(template[,3])
meta_time <- paste0("orig.date=", Year, "-", Month, "-", Day)
dates <- as.Date(paste(Year,Month,Day,sep="-")) + 10 # Conversion from Julian to Gregorian calendar
Year <- as.integer(substr(dates,1,4))
Month <- as.integer(substr(dates,6,7))
Day <- as.integer(substr(dates,9,10))
hours <- rep(NA,length(Month))
minutes <-rep(NA,length(Month))

###### Precipitation ########

dat <- round(as.numeric(template[,7])*25.4,2)
vardat <- dat
out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0("orig=",template[,4]," Pounds ", template[,5]," Ounces ",template[,6]," Grains")

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = paste(meta_time, meta_col, sep="|"),
          period = c(rep("7day",nrow(out)-1), "2day"),
          time_offset = 0,
          keep_na = TRUE,
          note="weekly")


## CRANE COURT ##
## NOTE: Temperature (Hawksby's scale) was not formatted

station_code <- "Europe_London_3"

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## Read template
template <- readWorksheetFromFile("../../3_Revised/London/Europe_T3_GB_London_Crane-Court_1729-1735_monthly.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template <- template[which(!is.na(template[,2])), ]
template[,1] <- fill_variable(template[,1], NA)
metadata <- inventory[which(inventory$Code == station_code), ]

## Mean pressure
pres <- convert_pressure(as.numeric(template[,4]), 25.4, as.numeric(metadata$Latitude),
                         as.numeric(metadata$Altitude))
out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = rep("", nrow(template)),
                  Hour = rep("", nrow(template)),
                  Minute = rep("", nrow(template)),
                  Value = round(pres, 1),
                  Meta = paste0("orig=", template[,4], "in"),
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N | Obs.times=morning | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

## Precipitation
out <- data.frame(Year = as.integer(template[,1]),
                  Month = as.integer(template[,2]),
                  Day = rep("", nrow(template)),
                  Hour = rep("", nrow(template)),
                  Minute = rep("", nrow(template)),
                  Value = round(as.numeric(template[,11])*25.4, 1),
                  Meta = paste0("orig=", template[,11], "in"),
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
