## NOTES:
## Temperature was not formatted (unknown scale, perhaps Florentine)

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## Read data
template <- readWorksheetFromFile("../../3_Revised/Lund/Europe_T3_SE_Lund_1724-1731_monthly.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code), ]
template <- template[which(!is.na(template[,2])), ]
template[,1] <- fill_variable(template[,1], NA)
n <- nrow(template)
for (j in 4:11) { # add leading zeros
  k <- which(nchar(template[25:n,j])==1)
  template[k+24,j] <- paste0("0", template[k+24,j])
}

## Mid-range pressure
pres <- (as.numeric(template[1:24,4]) + as.numeric(template[1:24,5])/10 +
           as.numeric(template[1:24,10]) + as.numeric(template[1:24,11])/10) / 2
pres <- append(pres, (as.numeric(template[25:36,4]) + as.numeric(template[25:36,5])/100 +
                       as.numeric(template[25:36,10]) + as.numeric(template[25:36,11])/100) / 2)
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))

out <- data.frame(Year = as.integer(template[1:36,1]),
                  Month = as.integer(template[1:36,2]),
                  Day = rep("", 36),
                  Hour = rep("", 36),
                  Minute = rep("", 36),
                  Value = round(pres, 1),
                  Meta = paste0("orig=", template[1:36,4], ".", template[1:36,5], "+", 
                                template[1:36,10], ".", template[1:36,11], "in"),
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

## Mean pressure
pres <- as.numeric(template[37:n,6]) + as.numeric(template[37:n,7])/100
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))

out <- data.frame(Year = as.integer(template[37:n,1]),
                  Month = as.integer(template[37:n,2]),
                  Day = rep("", 36),
                  Hour = rep("", 36),
                  Minute = rep("", 36),
                  Value = round(pres, 1),
                  Meta = paste0("orig=", template[37:n,6], ".", template[37:n,7], "in"),
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, 
                            " | PTC=N | PGC=Y | Obs.times=morning | Calendar=Julian"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub(" ", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
