## 

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) { 
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      } 
    }
  }
  return(timeseries)
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/Metz/Europe_T3_FR_Metz_1825-1830_monthly-pressure.xlsx", 
                                  sheet=1, startRow=7, header=FALSE, colTypes="character")

## Time
days <- hours <- minutes <- rep("", nrow(template))
template[,1] <- fill_variable(template[,1], NA)
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)

## Pressure
pres <- apply(template[,3:5], 1, function(x) mean(as.numeric(x)))
out$p <- round(convert_pressure(pres,1,as.numeric(metadata$Latitude),as.numeric(metadata$Altitude)), 2)
out$p_meta <- paste0("orig=", template[,3], "+", template[,4], "+", template[,5], "mmHg")

## Write SEF file
out <- out[which(!is.na(out$month)), ]

write_sef(out[, c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=Y | PGC=Y | Obs.times=9,12,15"),
          meta = out$p_meta,
          period = "month",
          note = "monthly")


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}