## 

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code & inventory$Source == "Giovene"), ]

## Read data
files <- list.files("../../3_Revised/Molfetta/", full.names=TRUE)
template <- readWorksheetFromFile(files, sheet=1, startRow=7, header=FALSE, colTypes="character")

## Time
days <- hours <- minutes <- rep("", 12*nrow(template))

## Precipitation
out <- data.frame(year = rep(as.integer(template[,1]), 12), 
                  month = rep(1:12, each=nrow(template)), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)
prec1 <- c(template[,seq(2,35,3)], recursive=TRUE)
prec2 <- c(template[,seq(3,36,3)], recursive=TRUE)
prec3 <- c(template[,seq(4,37,3)], recursive=TRUE)
prec <- as.numeric(prec1) + as.numeric(prec2)/12 + as.numeric(prec3)/144
out$rr <- round(prec*27.07, 1)
out$rr_meta <- paste0("orig=", prec1, ".", prec2, ".", prec3, "Pin")

## Write SEF file
out <- out[order(out$year,out$month), ]
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
