## NOTE: Temperature was not formatted (unknown/ambiguous scale) 
## Finished by Yuri

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)

## Read data
template <- readWorksheetFromFile("../../3_Revised/Montpellier/Europe_T3_FR_Montpellier_1705-1748_subdaily.xls",
                                  sheet=1, startRow=7, header=T, colTypes="character")

conversions <- list(p  = function(x) round(convert_pressure(as.numeric(x), f = 27.07, lat = lat, alt = alt), 1),
                    dd = function(x) 22.5 * (match(toupper(x), directions) - 1))

### Time ###
time_offset <- as.numeric(lon)*12/180
Year <- as.integer(fill_val(template[,"Year"]))
Month <- as.integer(fill_val(template[,"Month"]))
Day <- as.integer(fill_val(template[,"Day"]))
hours <- as.integer(substr(template[,"Time"],12,13))
minutes <- as.integer(substr(template[,"Time"],15,16))
meta_time <- paste0("orig.time=", substr(template[,"Time"],12,16)) 

###### Pressure ########
dat <- as.numeric(template[,8]) + as.numeric(template[,9])/12
vardat <- conversions[["p"]](dat)
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
neworder <-order(out$Year,out$Month,out$Day,out$hours,out$minutes)
meta_col <- paste0(meta_time, "|orig=", template[,8], ".", template[,9], "Pin")
write_sef(out[neworder,],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " | PGC=Y | PTC=N"),
          meta = meta_col[neworder],
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)

###### Wind direction ########
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", 
                "SSO", "SO", "OSO", "O", "ONO", "NO", "NNO")
vardat <- conversions[["dd"]](template[,10])
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
meta_col <- paste0(meta_time, "|orig=", template[,10])
write_sef(out[neworder,],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col[neworder],
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
