## NOTE: Cloud cover was not formatted being based on only three categories (clear, cloudy, overcast)

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/Pecs/Europe_T3_HU_Pecs_1819-1832_monthly.xlsx", 
                                  sheet=1, startRow=8, header=FALSE, colTypes="numeric")

## Time
days <- hours <- minutes <- rep("", 12*14)
out <- data.frame(year = rep(as.integer(template[2:15,1]), each=12), 
                  month = rep(1:12, 14), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)

## Temperature
template[2:15,2:13] <- t(t(as.matrix(template[2:15,2:13])) + c(template[1,2:13],recursive=TRUE))
temp <- c(t(template[2:15,2:13]), recursive=TRUE)
out$ta <- round(temp*1.25, 1)
out$ta_meta <- paste0("orig=", temp, "R")

## Wet days
out$nr <- c(t(template[2:15,30:41]), recursive=TRUE)

## Write SEF file
out <- out[order(out$year,out$month), ]

write_sef(out[, c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "month",
          note = "monthly")

write_sef(out[, c("year","month","day","hour","minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "days",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month")


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}