## 

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
template <- readWorksheetFromFile("../../3_Revised/Resterhafe-Osteel/Europe_T3_DE_Resterhafe-Osteel_1586-1612_monthly.xls", 
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
days <- hours <- minutes <- rep("", nrow(template))
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)

## Wet days
out$nr <- as.numeric(template[,5])
out$nr_meta <- paste0("orig=", template[,5])

## Write SEF file
write_sef(out[, c("year","month","day","hour","minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "days",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$nr_meta,
          period = "month",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}