## NOTES:

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## 1734-1741

## Read templates
template <- readWorksheetFromFile("../../3_Revised/Rome/Europe_T3_IT_Rome_1734-1735_monthly-prec.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template[,1] <- fill_variable(template[,1], NA)
template2 <- readWorksheetFromFile("../../3_Revised/Rome/Europe_T3_IT_Rome_1741_monthly.xlsx",
                                   sheet=1, startRow=8, header=FALSE, colTypes="character")

metadata <- inventory[which(inventory$Code == paste0(station_code,"_1")), ]

## Precipitation
prec <- as.numeric(c(template[,3], template2[,7])) * 25.4
meta_prec <- c(paste0("orig=", template[,3], "in"),
               paste0("orig=", template2[,7], "in"))

out <- data.frame(Year = as.integer(c(template[,1],template2[,1])),
                  Month = as.integer(c(template[,2],template2[,2])),
                  Day = rep("", nrow(template)+nrow(template2)),
                  Hour = rep("", nrow(template)+nrow(template2)),
                  Minute = rep("", nrow(template)+nrow(template2)),
                  Value = round(prec, 1),
                  Meta = meta_prec,
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = "month",
          note = "monthly",
          keep_na = TRUE)

## Pressure
pres <- (as.numeric(template2[,3]) + as.numeric(template2[,4])/12 + 
  as.numeric(template2[,5]) + as.numeric(template2[,6])/12) / 2
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude),
                         as.numeric(metadata$Altitude))
meta_pres <- paste0("orig=", template2[,3], ".", template2[,4], "+",
                    template2[,5], ".", template2[,6], "in")

out <- data.frame(Year = as.integer(template2[,1]),
                  Month = as.integer(template2[,2]),
                  Day = rep("", nrow(template2)),
                  Hour = rep("", nrow(template2)),
                  Minute = rep("", nrow(template2)),
                  Value = round(pres, 1),
                  Meta = meta_pres,
                  stringsAsFactors = FALSE)

write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, "| PGC=Y | PTC=N"),
          period = "month",
          note = "monthly",
          keep_na = TRUE)


file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub(" ", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)