## 

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)

template <- readWorksheetFromFile("../../3_Revised/Salzburg/Europe_T3_AT_Salzburg_1797-1798_subdaily.xls",
                                  sheet=1, startRow=8, header=F, colTypes="character")

atb <- rep(12.5, nrow(template))
conversions <- list(ta = function(x) round(as.numeric(x) * 1.25, 1),
                    p  = function(x) round(convert_pressure(as.numeric(x), f=27.07/12,
                                                            lat=lat, alt=alt, atb=atb), 1))


### Time ###
time_offset <- lon*12/180
Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- as.integer(template[,3])
hours <- as.integer(template[,4])
minutes <- as.integer(60 * (as.numeric(template[,4]) - hours))
hours[grep("s",template[,5])] <- hours[grep("s",template[,5])] + 12
meta_time <- paste0("orig.time=", template[,4], template[,5])


## Pressure
pres <- conversions[["p"]](template[,6])
out <- data.frame(Year,Month,Day,hours,minutes,pres)
meta_col <- paste0(meta_time,"|orig=",template[,6],"Pl|atb=10R")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|PTC=Y|PGC=Y"),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


## Temperature
temp <- conversions[["ta"]](template[,7])
out <- data.frame(Year,Month,Day,hours,minutes,temp)
meta_col <- paste0(meta_time,"|orig=",template[,7],"R")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


## Relative humidity
rh <- as.numeric(template[,10])
out <- data.frame(Year,Month,Day,hours,minutes,rh)
meta_col <- meta_time
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rh",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "%",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "| Instrument=Saussure | Note=Reduced to 10R"),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


## Precipitation type
rrt <- template[,11]
rrt <- sub("N", "F", rrt)
rrt <- sub(" ", "", rrt)
out <- data.frame(Year,Month,Day,hours,minutes,rrt)
meta_col <- paste0(meta_time,"|orig=",template[,11])
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rrt",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "coded",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = FALSE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
