## NOTE: Barometer 2 not formatted

library(dataresqc)
library(XLConnect)
library(lubridate)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)

template <- readWorksheetFromFile("../../3_Revised/Strasbourg/Europe_T3_FR_Strasbourg_1769-1772_subdaily.xlsx",
                                  sheet=1, startRow=8, header=F, colTypes="character")

conversions <- list(ta = function(x) round(as.numeric(x) * 1.25, 1),
                    p  = function(x) round(convert_pressure(as.numeric(x),f = 27.07,lat = lat,alt = alt), 1))

### Time ###
time_offset <- lon*12/180
Day <- as.integer(fill_val(template[,3]))
template <- template[which(!is.na(Day)), ]
Year <- as.integer(fill_val(template[,1]))
Month <- as.integer(fill_val(template[,2]))
Day <- as.integer(fill_val(template[,3]))
hours <- as.integer(template[,5])
minutes <- as.integer(60 * (as.numeric(template[,5]) - hours))
hours[grep("p",template[,6])] <- hours[grep("p",template[,6])] + 12
dates <- ISOdate(Year, Month, Day)
for (i in 1:5) {
  ddiff <- c(0, diff(dates))
  hdiff <- c(1, diff(hours))
  hours[which(ddiff==0 & hdiff<=0)] <- hours[which(ddiff==0 & hdiff<=0)] + 12
}
template[which(is.na(template[,6])), 6] <- ""
meta_time <- paste0("orig.date=", substr(dates,1,10), "|orig.time=", template[,5], template[,6])
dates[which(hours==24)] <- dates[which(hours==24)] + 24*3600


## Pressure
pres <- as.numeric(template[,7]) + as.numeric(template[,8])/12
pres <- conversions[["p"]](pres)
out <- data.frame(year(dates),month(dates),day(dates),hours,minutes,pres)
meta_col <- paste0(meta_time,"|orig=",template[,7],".",template[,8],"Pin")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|PTC=N|PGC=Y"),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


## Temperature
temp <- conversions[["ta"]](template[,9])
out <- data.frame(year(dates),month(dates),day(dates),hours,minutes,temp)
meta_col <- paste0(meta_time,"|orig=",template[,9],"R")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


## Precipitation type
rrt <- template[,4]
rrt <- sub("p", "R", rrt)
rrt <- sub("n", "S", rrt)
rrt <- sub(" ", "", rrt)
out <- data.frame(year(dates),month(dates),day(dates),hours,minutes,rrt)
meta_col <- paste0(meta_time,"|orig=",template[,4])
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rrt",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "coded",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = FALSE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
