## NOTE: Temperature was not formatted (insufficient information on thermometer)

library(dataresqc)
library(XLConnect)


## Create output directory (if necessary) and station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}
station_code <- paste(name_group, name_folder, sep = "_")


## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code), ]


## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}


## Read data
template <- readWorksheetFromFile(list.files("../../3_Revised/Trondheim",full.names=TRUE),
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
years <- as.integer(template[,1])
months <- as.integer(template[,2])
days <- as.integer(template[,3])
hours <- rep(NA, nrow(template))
minutes <- rep(NA, nrow(template))

## Pressure
template[,4] <- fill_variable(template[,4], NA)
pres <- as.numeric(template[,4]) + as.numeric(template[,5]) / 12
pres <- convert_pressure(pres, 27.07, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
meta_col <- paste0("orig=", template[,4], ".", template[,5], "Pin") 
out <- data.frame(Year = years,
                  Month = months,
                  Day = days,
                  Hour = hours,
                  Minute = minutes,
                  Value = round(pres, 1),
                  Meta = meta_col,
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          period = 0,
          keep_na = TRUE)

## Wind
directions <- c("N", "NNO", "NO", "ONO", "O", "OSO", "SO", "SSO", "S",
                "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")
wdir <- 22.5 * (match(toupper(template[,7]), directions) - 1)
meta_col <- paste0("orig=", template[,7]) 
out <- data.frame(Year = years,
                  Month = months,
                  Day = days,
                  Hour = hours,
                  Minute = minutes,
                  Value = round(wdir, 0),
                  Meta = meta_col,
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = 0,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
