## Pressure for 1691-1694 was not formatted (unknown scale)

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}

## Create output directory (if necessary)
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")


## Camerarius (1691-1694)
station_code <- paste(name_group, name_folder, 1, sep = "_")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
template <- readWorksheetFromFile("../../3_Revised/Tuebingen/Europe_T3_DE_Tuebingen_1691-1694_subdaily.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
Year <- as.integer(fill_val(template[,1]))
Month <- as.integer(fill_val(template[,2]))
Day <- as.integer(fill_val(template[,3]))
hours <- as.integer(substr(template[,4],12,13))
minutes <- as.integer(substr(template[,4],15,16))
meta_time <- template[,4]
j <- which(nchar(meta_time)>1)
meta_time[j] <- substr(meta_time[j], 12, 16)
meta_time <- paste0("orig.time=", meta_time)

## Temperature (from Lenke)
temp <- as.numeric(template[,7])
meta_col <- paste0(meta_time, "|orig=", template[,7], "C")
out <- data.frame(Year, Month, Day, hours, minutes, temp)

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)


## Kraft (1745-1746)
station_code <- paste(name_group, name_folder, 2, sep = "_")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "monthly"), ]
lat <- as.numeric(metadata$Latitude)
alt <- as.numeric(metadata$Altitude)
template <- readWorksheetFromFile("../../3_Revised/Tuebingen/Europe_T3_DE_Tuebingen_1745-1746_monthly.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- rep("", nrow(template))
hours <- Day
minutes <- Day

## Temperature
temp <- round((as.numeric(template[,6]) - 32) * 5/9, 1)
meta_col <- paste0("orig=", template[,6], "F")
out <- data.frame(Year, Month, Day, hours, minutes, temp)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = "month",
          note = "monthly-max",
          keep_na = TRUE)

temp <- round((as.numeric(template[,7]) - 32) * 5/9, 1)
meta_col <- paste0("orig=", template[,7], "F")
out <- data.frame(Year, Month, Day, hours, minutes, temp)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = "month",
          note = "monthly-min",
          keep_na = TRUE)

## Pressure
pres <- round(convert_pressure(as.numeric(template[,3]),25.4,lat,alt), 1)
meta_col <- paste0("orig=", template[,3], "Rh.in")
out <- data.frame(Year, Month, Day, hours, minutes, pres)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          meta = meta_col,
          period = "month",
          note = "monthly-max",
          keep_na = TRUE)

pres <- round(convert_pressure(as.numeric(template[,4]),25.4,lat,alt), 1)
meta_col <- paste0("orig=", template[,4], "Rh.in")
out <- data.frame(Year, Month, Day, hours, minutes, pres)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          meta = meta_col,
          period = "month",
          note = "monthly-min",
          keep_na = TRUE)

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
