## 

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]

## Temperature (from Lenke)
template <- readWorksheetFromFile("../../3_Revised/Ulm/Europe_T3_DE_Ulm_1710-1714_subdaily_temp.xlsx",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

Year <- as.integer(fill_val(template[,1]))
Month <- as.integer(fill_val(template[,2]))
Day <- as.integer(fill_val(template[,3]))
hours <- rep(NA, nrow(template))
minutes <- hours
meta_time <- paste0("orig.time=", template[,4])

temp <- as.numeric(template[,6])
meta_col <- paste0(meta_time, "|orig=", template[,6], "C")
out <- data.frame(Year, Month, Day, hours, minutes, temp)

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          keep_na = TRUE)


## Precipitation
template <- readWorksheetFromFile("../../3_Revised/Ulm/Europe_T3_DE_Ulm_1715-1721_daily_precip.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "daily"), ]

Year <- as.integer(fill_val(template[,1]))
Month <- as.integer(fill_val(template[,2]))
Day <- as.integer(fill_val(template[,3]))

## Add zeros
dates <- as.Date(paste(Year,Month,Day,sep="-"))
all_dates <- seq(as.Date("1715-03-24"), as.Date("1721-03-05"), by="days")
Year <- as.integer(substr(all_dates,1,4))
Month <- as.integer(substr(all_dates,6,7))
Day <- as.integer(substr(all_dates,9,10))
prec <- rep(0, length(Year))
meta_col <- rep("", length(Year))

j <- match(dates, all_dates)
prec[j] <- round(as.numeric(template[,4]) * 25.33 / 192, 1)
meta_col[j] <- paste0("orig=", template[,4], "Nur.in/192")

hours <- rep(NA, length(Year))
minutes <- hours
out <- data.frame(Year, Month, Day, hours, minutes, prec)

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = "day",
          keep_na = TRUE)


## Precipitation type
rrt <- rep("R", nrow(template))
rrt[grep("n", template[,5])] <- "S"
meta_col <- paste0("orig=",template[,5])

Year <- as.integer(fill_val(template[,1]))
Month <- as.integer(fill_val(template[,2]))
Day <- as.integer(fill_val(template[,3]))
hours <- rep(NA, nrow(template))
minutes <- hours

out <- data.frame(Year,Month,Day,hours,minutes,rrt)

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rrt",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "coded",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
