## 

library(dataresqc)
library(XLConnect)



## Create output directory (if necessary) and station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}
station_code <- paste(name_group, name_folder, sep = "_")

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code), ]

## Define function to fill missing values with the value on the previous row
fill_variable <- function(timeseries, strings) {
  for (i in 2:length(timeseries)) {
    if (timeseries[i] %in% strings) {
      if(!timeseries[i-1] %in% strings) {
        timeseries[i] <- timeseries[i-1]
      }
    }
  }
  return(timeseries)
}

## Read data
template <- readWorksheetFromFile("../../3_Revised/Upminster/Europe_T3_GB_Upminster_1708_daily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")

## Time
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
dates <- dates + 10 # convert from Julian to Gregorian calendar
years <- as.integer(format(dates,"%Y"))
months <- as.integer(format(dates,"%m"))
days <- as.integer(format(dates,"%d"))
hours <- rep(12, nrow(template))
minutes <- rep(0, nrow(template))
meta_time <- paste0("orig.date=", dates-10, "|orig.time=noon")

## Pressure
template[,6] <- fill_variable(template[,6], NA)
pres <- as.numeric(template[,6]) + as.numeric(template[,7]) / 100
pres <- convert_pressure(pres, 25.4, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude))
meta_col <- paste0(meta_time, "|orig=", template[,6], ".", template[,7], "in") 
out <- data.frame(Year = years,
                  Month = months,
                  Day = days,
                  Hour = hours,
                  Minute = minutes,
                  Value = round(pres, 1),
                  Meta = meta_col,
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer, " | PTC=N | PGC=Y"),
          period = 0,
          time_offset = as.numeric(metadata$Longitude)*12/180,
          keep_na = TRUE)

## Precipitation
template[which(is.na(template[,14])),14] <- 0
template[which(is.na(template[,15])),15] <- 0
prec <- 25.4 * (as.numeric(template[,14]) + as.numeric(template[,15])/100)
meta_time <- paste0("orig.date=", dates-10)
meta_col <- paste0(meta_time, "|orig=", template[,14], ".", template[,15], "in") 
out <- data.frame(Year = years,
                  Month = months,
                  Day = days,
                  Hour = rep(NA,nrow(template)),
                  Minute = rep(NA,nrow(template)),
                  Value = round(prec, 1),
                  Meta = meta_col,
                  stringsAsFactors = FALSE)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          meta = out$Meta,
          metaHead = paste0("Observer=", metadata$Observer),
          period = "day",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
