## NOTE: Temperature not formatted (unknown scale)

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, 1, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)

## Read data
files <- list.files("../../3_Revised/Vienna", full.names=TRUE)
out <- list()
for (i in 1:length(files)) {
  
  template <- readWorksheetFromFile(files[i], sheet=ifelse(i==1,3,1), startRow=8, 
                                    header=FALSE, colTypes="character")
  Year <- as.integer(fill_val(template[,1]))
  Month <- as.integer(fill_val(template[,2]))
  Day <- fill_val(as.integer(template[,3]))
  if (i == 1) {
    hours <- rep(NA, nrow(template))
    minutes <- hours
    meta_time <- "orig.time=NA"
  } else {
    hours <- as.integer(template[,4])
    minutes <- as.integer((as.numeric(template[,4]) - hours) * 60)
    meta_time <- paste0("orig.time=", template[,4])
    ## add 12 hours to PM observations
    for (k in 2:length(hours)) {
      if (!is.na(hours[k]) & !is.na(hours[k-1])) {
        if (Day[k] == Day[k-1] & hours[k] <= hours[k-1]) hours[k] <- hours[k] + 12
        if (k < length(hours)) { # assume last observation is always PM
          if (Day[k] == Day[k-1] & Day[k] != Day[k+1] & hours[k] < 12) hours[k] <- hours[k] + 12
        }
      }
    }
  }
  
  j <- ifelse(i==1, 4, 5)
  template[,j] <- fill_val(template[,j])
  pres <- as.numeric(template[,j]) + as.numeric(template[,j+1])/60
  pres <- round(convert_pressure(pres, f=26.15, lat, alt), 1)
  meta_col <- paste0(meta_time, "|orig=", template[,j], ".", template[,j+1], "Rh.in")
  
  out[[i]] <- data.frame(Year, Month, Day, hours, minutes, pres, meta_col)
  
}

## Write data to SEF
out <- Reduce(rbind, out)
write_sef(out[, 1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|PTC=N|PGC=Y"),
          meta = out$meta_col,
          period = 0,
          time_offset = lon*12/180,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
