## 

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, 1, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Read metadata
metadata <- inventory[which(inventory$Code == station_code), ]

## Read data
files <- list.files("../../3_Revised/Zurich/", full.names=TRUE)
template <- read.table(files, header=TRUE, sep="\t", stringsAsFactors=FALSE)


## MONTHLY ##

## Initialize data frame
days <- hours <- minutes <- rep("", nrow(template))
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = days, 
                  hour = hours, 
                  minute = minutes)

## Precipitation
out$rr <- as.numeric(template[,3])
out$rr_meta <- paste0("orig=", template[,3], "mm")


## Write SEF files
write_sef(out[, c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
#  print(f)
  check_sef(f)
}
