## NOTE: Stations with unknown coordinates are not formatted

library(dataresqc)
library(XLConnect)

files <- list.files("../../3_Revised/Dove/", pattern="Dove_Part", full.names=TRUE)

inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")

## Define units conversion
conv <- list(C = function(x) x,
             R = function(x) x*1.25,
             F = function(x) (x-32)*5/9)

## Read metadata
meta <- list()
for (i in 1:length(files)) {
  metadata <- readWorksheetFromFile(files[i], sheet=1, colTypes="character", header=FALSE)
  ids <- metadata[,1]
  stations <- metadata[,2]
  if (i == 1) {
#    lats <- as.numeric(metadata[,4]) + as.numeric(metadata[,5])/60
#    lons <- as.numeric(metadata[,7]) + as.numeric(metadata[,8])/60
#    lons[grep("-",metadata[,6])] <- -lons[grep("-",metadata[,6])]
    times <- metadata[,11]
    observers <- metadata[,12]
    calendar <- metadata[,13]
  } else if (i == 2) {
    metadata[which(is.na(metadata[,4])),4] <- 0
    metadata[which(is.na(metadata[,6])),6] <- 0
#    lats <- as.numeric(metadata[,3]) + as.numeric(metadata[,4])/60
#    lons <- as.numeric(metadata[,5]) + as.numeric(metadata[,6])/60
#    lons[c(1:130,174:200)] <- -lons[c(1:130,174:200)]
#    lons[c(131:160,162:173,201:233)] <- lons[c(131:160,162:173,201:233)] - 20 + 2.337
    times <- metadata[,9]
    observers <- metadata[,8]
    calendar <- metadata[,10]
  } else if (i == 3) {
    metadata[which(is.na(metadata[,4])),4] <- 0
    metadata[which(is.na(metadata[,7])),7] <- 0
#    lats <- as.numeric(metadata[,3]) + as.numeric(metadata[,4])/60
#    lons <- as.numeric(metadata[,6]) + as.numeric(metadata[,7])/60
#    lons[grep("-",metadata[,5])] <- -lons[grep("-",metadata[,5])]
#    lons[c(38:39,41:45,47)] <- lons[c(38:39,41:45,47)] - 20 + 2.337
    times <- rep(NA, nrow(metadata))
    times <- metadata[,10]
    observers <- metadata[,9]
    calendar <- metadata[,11]
  } else if (i == 4) {
    metadata[which(is.na(metadata[,5])),5] <- 0
    metadata[which(is.na(metadata[,8])),8] <- 0
#    lats <- as.numeric(metadata[,4]) + as.numeric(metadata[,5])/60
#    lats[grep("-",metadata[,3])] <- -lats[grep("-",metadata[,3])]
#    lons <- as.numeric(metadata[,7]) + as.numeric(metadata[,8])/60
#    lons[grep("-",metadata[,6])] <- -lons[grep("-",metadata[,6])]
    times <- rep(NA, nrow(metadata))
    times <- metadata[,11]
    observers <- metadata[,10]
    calendar <- metadata[,12]
  }
  ids[which(ids=="NA")] <- paste0(ids[which(ids=="NA")-1], "a")
  ids[grep("aa",ids)] <- sub("aa", "b", ids[grep("aa",ids)])

  ## Read coordinates from inventory
  lats <- inventory$Latitude[match(paste0("Global-Datasets_Dove-",ids),inventory$Code)]
  lons <- inventory$Longitude[match(paste0("Global-Datasets_Dove-",ids),inventory$Code)]

  meta[[i]] <- data.frame(ids, stations, lats, lons, times, observers, calendar, 
                          part=rep(i,length(ids)), stringsAsFactors=FALSE)
}
meta <- Reduce(rbind, meta)
meta$calendar <- sub("I2", "Interpolated (1/2 + 1/2)", meta$calendar)
meta$calendar <- sub("I3", "Interpolated (1/3 + 2/3)", meta$calendar)

## Read data
out <- list()
for (i in 1:length(files)) { # loop over files
  template <- readWorksheetFromFile(files[i], sheet=2, colTypes="character", header=FALSE)
  k <- 0 # initialize line number
  while (k < nrow(template)) { # loop over stations
    tmp <- rep(NA, 13)
    k <- k+1
    id <- ifelse(template[k,1]=="NA", paste0(id,"a"), template[k,1])
    if (grepl("aa",id)) id <- sub("aa", "b", id)
    unit <- template[k, 3]
    i_meta <- which(meta$ids==id & meta$part==i)
    if (length(i_meta) == 0) i_meta <- which(meta$ids==id)[1]
    k <- k+1
    while (!is.na(template[k,1])) { # loop over lines
      tmp <- rbind(tmp, template[k, 1:13])
      k <- k+1
    }
    if (!is.na(meta$lats[i_meta]) & !is.na(meta$lons[i_meta])) {
      years <- as.integer(tmp[,1])
      tmp <- tmp[which(!is.na(years) & !duplicated(years)), ]
      meta_col <- paste0("orig=", c(t(tmp[,2:13]), recursive=TRUE), unit)
#      if (!is.na(meta$calendar[i_meta])) meta_col <- paste0(meta_col, "|Calendar=", meta$calendar[i_meta])
      if (nrow(tmp) > 0) {
        print(paste(id, meta$stations[i_meta]), quote=FALSE)
        years <- as.integer(tmp[,1])
        if (!all(order(years) == (1:length(years)))) stop("Error in the years")
        temp <- conv[[unit]](as.numeric(c(t(tmp[,2:13]),recursive=TRUE)))
        if (id %in% names(out) & length(unique(meta$calendar[which(meta$ids==id)])) == 1) {
          out[[id]] <- rbind(out[[id]],
                             data.frame(year = rep(years, each=12),
                                        month = rep(1:12, length(years)),
                                        day = rep("", 12*length(years)),
                                        hour = rep("", 12*length(years)),
                                        minute = rep("", 12*length(years)),
                                        ta = round(temp, 1),
                                        meta = meta_col,
                                        calendar = meta$calendar[i_meta]))
        } else {
          if (length(unique(meta$calendar[which(meta$ids==id)])) > 1) id <- paste0(id, "c")
          out[[id]] <- data.frame(year = rep(years, each=12),
                                  month = rep(1:12, length(years)),
                                  day = rep("", 12*length(years)),
                                  hour = rep("", 12*length(years)),
                                  minute = rep("", 12*length(years)),
                                  ta = round(temp, 1),
                                  meta = meta_col,
                                  calendar = meta$calendar[i_meta])
        }
        if (id %in% c("295","379") & i==3) out[[id]] <- NULL # skip Archangelsk and Nizhney Tagilsk from Dove 3
      }
    }
  }
}

## Write SEF files
for (id in names(out)) {
  i_meta <- which(meta$ids==sub("c","",id))
  observers <- paste(unique(meta$observers[i_meta]), collapse=",")
  times <- paste(unique(meta$times[i_meta]), collapse=";")
  meta_head <- paste0("Observer=", observers, " | Obs.times=", times)
  if (!is.na(out[[id]]$calendar[1])) meta_head <- paste0(meta_head, " | Calendar=", out[[id]]$calendar[1])
  i_meta <- i_meta[1]
  out[[id]] <- out[[id]][order(out[[id]]$year,out[[id]]$month), ]
  dates <- paste0(out[[id]]$year, "-", out[[id]]$month)
  i_dupl <- which(duplicated(dates))
  ko <- c()
  for (k in i_dupl) { # remove duplicates keeping the latest version (if not NA)
    ko <- append(ko, ifelse(is.na(out[[id]]$ta[k]),k,k-1))
  }
  if (length(ko) > 0) out[[id]] <- out[[id]][-ko, ]
  write_sef(out[[id]][,1:6], "../../4_Formatted/Dove/", "ta", paste0("Global-Datasets_Dove-",meta$ids[i_meta]), 
            meta$stations[i_meta], meta$lats[i_meta], meta$lons[i_meta], alt=NA, "PALAEO-RA", 
            link=NA, "C", "mean", meta_head, out[[id]]$meta, "month", note="monthly", keep_na=TRUE)
}

## Check SEF files
#for (f in list.files("../../4_Formatted/Dove/", full.names=TRUE)) {
#  print(f)
#  check_sef(f)
#}
