library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")

## Read data and metadata
template <- readWorksheetFromFile("../../3_Revised/Boston/USA_T3_US_Boston_Charlestown_1792-1802_monthly-prec.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly"), ]

## Create data frame with SEF structure
out <- data.frame(Year = rep(as.integer(template[,1]), each=12),
                  Month = rep(1:12, nrow(template)),
                  Day = rep(NA, nrow(template)*12),
                  Hour = rep("", nrow(template)*12),
                  Minute = rep("", nrow(template)*12),
                  Value = round(25.4 * as.numeric(c(t(template[,2:13]))), 1),
                  Meta = paste0("orig=", c(t(template[,2:13])), "in"),
                  stringsAsFactors = FALSE)

## Write SEF file
write_sef(out[,1:6],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$Meta,
          period = "month",
          note = "monthly",
          keep_na = TRUE)

file.rename(list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE), 
            gsub("NA", "", list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)))

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)