## NOTES:
## -
#rm(list=ls())

library(dataresqc)
library(XLConnect)

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                 sheet=1, colTypes="character")


## SUBDAILY
template_0714 <- readWorksheetFromFile("../../3_Revised/Brunswick/USA_T3_US_Brunswick_1807-1814_subdaily.xls",
                                  sheet=1, startRow=8, endCol=23, header=T, colTypes="character")
template_1522 <- readWorksheetFromFile("../../3_Revised/Brunswick/USA_T3_US_Brunswick_1815-1822_subdaily.xls",
                                       sheet=1, startRow=8, endCol=21, header=T, colTypes="character")
## remove min max for merging
template <- rbind(template_0714[,-c(7,8)], template_1522) 
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)

############################
## Daily
############################

## Temperature (Fahrenheit)
## Min Max
dates_stat <- as.Date(paste(template_0714[,1], template_0714[,2], template_0714[,3], sep="-"))
dates_stat <- dates_stat[1:which(dates_stat=="1808-01-31")]
template_stat <- template_0714[,c(1,2,3,7,8)]
template_stat <- template_stat[1:length(dates_stat),]

hours_stat <- rep(NA, times = rep(nrow(template_stat)))
minutes_stat <- rep(NA, times=length(hours_stat))
#meta_time_stat <- paste0("orig.date=", dates_stat)

for(tt in c("Min","Max")){
  temp <- round((as.numeric(template_stat[,tt]) - 32) * 5 / 9, 1)
  out <- data.frame(as.integer(template_stat[,1]), 
                    as.integer(template_stat[,2]), 
                    as.integer(template_stat[,3]), 
                    hours_stat, minutes_stat, temp)
  #meta_col <- paste0(meta_time_stat, "|orig=", template_stat[,tt], "F")
  meta_col <- ifelse(is.na(template_stat[,tt]),paste0("orig=", template_stat[,tt]),paste0("orig=", template_stat[,tt], "F"))
  write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = switch((tt=="Min")+1,"Tx","Tn"),
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = "C",
            stat = paste0(tolower(tt),"imum"),
            metaHead = ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),
            meta = meta_col,
            period = "day",
            time_offset = as.numeric(metadata$Longitude) * 12 / 180,
            keep_na = TRUE)
}

## Precipitation
dates_prec <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
hours_prec <- rep(NA, times = rep(length(dates_prec)))
minutes_prec <- rep(NA, times=length(hours_prec))
#meta_time_prec <- paste0("orig.date=", dates_prec)

for(tt in c("Inches.of.rain","Inches.of.snow")){
  
  dat <- template[,tt]
  if(length(dat[grepl("|",dat,fixed = T)])>0){message("kumuliert")}

  
  dat[is.na(dat)] <- 0
  allna <- apply(template[,4:8],1,function(x){all(is.na(x))})
  dat[allna] <- NA
  if(tt=="Inches.of.snow"){
    if(length(dat[grepl("or",dat,fixed = T)])>0){
      message(dat[grepl("or",dat,fixed = T)])
      dat[grepl("or",dat,fixed = T)] <- c("4.5","4.5","13")
    }
  }
  dat_conv <- round((as.numeric(dat) *25.4), 1)
  
  out <- data.frame(as.integer(template[,1]), 
                    as.integer(template[,2]), 
                    as.integer(template[,3]), 
                    hours_prec, minutes_prec, dat_conv)
  meta_col <- ifelse(is.na(dat),paste0("orig=", dat),paste0("orig=", dat, "in"))
  write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = switch((tt=="Inches.of.rain")+1,"fs","rr"),
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = "mm",
            stat = "sum",
            metaHead = ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),
            meta = meta_col,
            period = "day",
            time_offset = as.numeric(metadata$Longitude) * 12 / 180,
            keep_na = TRUE)
}


################################################
### Subdaily
################################################

directions <- c("N", "NE", "E", "SE", "S", "SW","W", "NW")
prec_type <- c("R","S","SLEET","FOG","MIST")
prec_nam <- c("R","S","S","F","M")

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    p  = function(x) round(dataresqc::convert_pressure(as.numeric(x), 
                                                            f = 25.4, 
                                                            lat = lat, 
                                                            alt = alt), 1),
                    dd = function(x) {return(45 * (match(toupper(x), directions) - 1))},
                    rrt = function(x) return(prec_nam[match(x, prec_type)]))

units <- list(ta = "C",
              p  = "hPa",
              dd = "degree",
              rrt = "Coded")


### time ###
Year <- rep(as.integer(template[,1]), each=3)
Month <- rep(as.integer(template[,2]), each=3)
Day <- rep(as.integer(template[,3]), each=3)
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- rep(NA,length(Year))
minutes <- rep(NA, length(Year))

time_offset <- as.numeric(lon)*12/180

##sunset
utc_ss <- suncalc::getSunlightTimes(date=dates[seq(3,length(dates),3)],lat=lat,lon=lon, keep="sunset")[,4]+(15*60)
Year[seq(3,length(hours),3)] <- substr(utc_ss,1,4)
Month[seq(3,length(hours),3)] <- substr(utc_ss,6,7)
Day[seq(3,length(hours),3)] <- substr(utc_ss,9,10)
hours[seq(3,length(hours),3)] <- substr(utc_ss,12,13)
minutes[seq(3,length(hours),3)] <- substr(utc_ss,15,16)

#morning + noon
ind <- c(1:length(Year))[(1:length(Year)%%3!=0)]
utc_mono <- ISOdate(Year[ind],Month[ind],Day[ind],rep(c(7,13),length.out=length(ind)),rep(c(30,0),length.out=length(ind)),tz="") - time_offset*3600
Year[ind] <- substr(utc_mono,1,4)
Month[ind] <- substr(utc_mono,6,7)
Day[ind] <- substr(utc_mono,9,10)
hours[ind] <- substr(utc_mono,12,13)
minutes[ind] <- substr(utc_mono,15,16)

meta_time <- paste0("orig.date=",dates,"|orig.time=",rep(c("morning","noon","night"), length.out=length(Year)))
#meta_time <- paste0("orig.time=",rep(c("morning","noon","night"), length.out=length(Year)))
startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

for(var in c("ta","p","dd","rrt")){
    
  # data
  var_col <- switch(var,"ta"=c(4:6),"p"=c(7:9),"dd"=c(10:12),"rrt"=c(16:18))
  orig_u <- switch(var,"ta"="F","p"="in","dd"="","rrt"="")  
  dat <- matrix(t(as.matrix(template[,var_col])),ncol=1)
  
  if(var=="rrt"){
    repl <- c("Rain","Snow","Sleet","Fog","Mist")[match(dat, prec_type)]
    orig_val <- repl
  } else{
    orig_val <- dat
  }
  
  dat[grepl(pattern="[",x=dat, fixed=T)] <- NA
  vardat <- conversions[[var]](dat)
  
  out <- data.frame(Year, Month, Day, hours, minutes, vardat, stringsAsFactors=FALSE)
  
  meta_col <- paste0(meta_time, ifelse(is.na(orig_val),paste0("|orig=",orig_val),paste0("|orig=",orig_val,orig_u)))
  sep <- ifelse(!is.na(metadata$Observer) & var=="p","|","")
  write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = var,
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = units[[var]],
            stat = "point",
            metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),sep,switch(var,"p"="PTC=N|PGC=Y","")),
            meta = meta_col,
            period = 0,
            time_offset = 0,
            outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_",var,".tsv"),
            keep_na = TRUE)
  
}


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
