#rm(list=ls())

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}

## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
##### Altitude fehlt
alt <- as.numeric(metadata$Altitude)


############################
## subdaily
############################
template <- readWorksheetFromFile("../../3_Revised/Concord/USA_T3_US_Concord_1775-1776_subdaily.xls",
                                  sheet=1, startRow=7, header=T, colTypes="character")
#View(template)

directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    dd = function(x) {return(22.5 * (match(toupper(x), directions) - 1))},
                    wind_force  = function(x) {as.numeric(x)}
                    )

### time ###
time_offset <- as.numeric(lon)*12/180

Year <- as.integer(template[,"Year"])
Month <- as.integer(template[,"Month"])
Day <- as.integer(template[,"Day"])
dates <- as.Date(paste(Year,Month,Day, sep="-"))

Time <- template[,"Time"]
## on every day two measurements, thus simply add 12h
hours <- floor(as.numeric(Time))+rep(c(0,12),times=length(Time)/2)
minutes <- 60 * (as.numeric(Time)- floor(as.numeric(Time)))
orig_h <- Time

# create artifical hours for missing hours, but only in order to order the dates 
nahour <-which(is.na(hours))
hours2 <- hours
min2 <- minutes
hours2[nahour] <- ifelse((nahour%%2)==1,8,15)
min2[nahour] <- 0
dates_help <- paste0(dates, " ", ClimIndVis:::pad2(hours2),":",ClimIndVis:::pad2(min2))

## order dates
new_order <- order(dates_help)
Year <- Year[new_order]
Month <- Month[new_order]
Day <- Day[new_order]
hours <- hours[new_order]
minutes <- minutes[new_order]
dates <- dates[new_order]
orig_h <- orig_h[new_order]

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)
startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],
                   ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

############## 
#### Temp F
############## 
dat <- template[new_order,8]
orig_val <- dat
vardat <- conversions[["ta"]](dat)
  
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
#View(out)  

meta_col <- paste0(meta_time, ifelse(orig_val=="NA",paste0("|orig=",orig_val),paste0("|orig=",orig_val,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_ta.tsv"))
  

############## 
####  Wind dir
############## 
dat <- template[new_order,9]
unique(dat)
vardat <- conversions[["dd"]](dat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time,"|orig=",dat)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_dd.tsv"))


############## 
####  Wind Force
############## 
dat <- template[new_order,10]
unique(dat)
vardat <- conversions[["wind_force"]](dat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time,"|orig=",dat)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "wind_force",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = NA,
          stat = "point",
          metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_wind_force.tsv"))



## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)

