#rm(list=ls())

library(dataresqc)
library(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls",
                                   sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
##### Altitude fehlt
alt <- as.numeric(metadata$Altitude)


############################
## subdaily
############################
template <- readWorksheetFromFile("../../3_Revised/Ipswich/USA_T3_US_Ipswich_1781-1783_subdaily.xls",
                                       sheet=1, startRow=7, header=T, colTypes="character")

directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    p  = function(x) {return(round(convert_pressure(as.numeric(x),f = 25.4,lat = lat,alt = alt), 1))},
                    dd = function(x) {return(22.5 * (match(toupper(x), directions) - 1))})

units <- list(ta = "C",p  = "hPa",dd = "degree")

### time ###
time_offset <- as.numeric(lon)*12/180

Year <- as.integer(fill_val(template[,"Year"]))
Month <- as.integer(fill_val(template[,"Month"]))
Day <- as.integer(fill_val(template[,"Day"]))
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- as.integer(substr(template[,"Hour"],12,13))
minutes <- as.integer(substr(template[,"Hour"],15,16))
orig_h <- substr(template[,"Hour"],12,19)
orig_h[orig_h==""] <- NA

## order dates
new_order <- order(dates)
Year <- Year[new_order]
Month <- Month[new_order]
Day <- Day[new_order]
hours <- hours[new_order]
minutes <- minutes[new_order]
dates <- dates[new_order]
orig_h <- orig_h[new_order]

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

for(var in c("ta","p","dd")){
  
  # data
  var_col <- switch(var,"ta"=c(5),"p"=c(6,7),"dd"=c(8))
  orig_u <- switch(var,"ta"="F","p"="in","dd"="")  
  dat <- template[new_order,var_col]
  if(var=="p"){
    dat1 <- fill_val(dat[,1])
    for(ii in 1:length(dat1)){
      if(!is.na(dat1[ii])){
        if(dat1[ii]=="NA"  & dat[ii,2]!="NA")  {
          last <- which(dat1[1:ii]!="NA")
          dat1[ii] <- dat1[last[length(last)]]
        }
      }
    }
    orig_val <- paste0(dat1,".",dat[,2])
    dat <- as.numeric(dat1) + as.numeric(dat[,2])/100
  } else {
    orig_val <- dat
  }
  
  #dat[grepl(pattern="[",x=dat, fixed=T)] <- NA
  vardat <- conversions[[var]](dat)
  
  out <- data.frame(Year,Month,Day,hours,minutes,vardat)
  
  meta_col <- paste0(meta_time, ifelse(is.na(orig_val),paste0("|orig=",orig_val),paste0("|orig=",orig_val,orig_u)))
  sep <- ifelse(!is.na(metadata$Observer) & var=="p","|","")
  write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = var,
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = units[[var]],
            stat = "point",
            metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),sep,switch(var,"p"="PTC=N|PGC=Y","")),
            meta = meta_col,
            period = 0,
            time_offset = time_offset,
            keep_na = TRUE)
  
}


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)


############################
## Monthly
############################
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "monthly"), ]

template <- readWorksheetFromFile("../../3_Revised/Ipswich/USA_T3_US_Ipswich_1781-1783_monthly.xls",
                                  sheet=1, startRow=7, header=T, colTypes="character")

#variables <- c("Tx","Tn","ta","p","p","p","rr")
variables <- c("ta","ta","ta","p","p","p","rr")
columns  <- c(3,4,5,6,7,8,9)
orig_units <- c("F","F","F","in","in","in","in")
stats <- c("maximum","minimum","mean","maximum","minimum","mean","sum")
conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    Tx = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    Tn = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    p  = function(x) {return(round(convert_pressure(as.numeric(x),f = 25.4,lat = lat,alt = alt), 1))},
                    rr = function(x) {round(as.numeric(x)*25.4,1)})

units <- c("C","C","C","hPa","hPa","hPa","mm")

### time ###
time_offset <- as.numeric(lon)*12/180

Year <- as.integer(template[,"Year"])
Month <- as.integer(template[,"Month"])
Day <- rep(NA,length(Month))
hours <- rep(NA,length(Month))
minutes <-rep(NA,length(Month))
startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2))

for(var in 1:length(variables)){
  var1 <- variables[var]
  var_col <- columns[var]
  orig_u <-  orig_units[var]
  dat <- template[,var_col]
  orig_val <- dat
  
  vardat <- conversions[[var1]](dat)
  
  out <- data.frame(Year,Month,Day,hours,minutes,vardat)
  
  meta_col <- paste0(ifelse(is.na(orig_val),paste0("orig=",orig_val),paste0("orig=",orig_val,orig_u)))
  sep <- ifelse(!is.na(metadata$Observer) & var1=="p","|","")
  
  write_sef(out,
            outpath = paste0("../../4_Formatted/", name_folder),
            variable = var1,
            cod = metadata$Code,
            nam = metadata$Station,
            lat = metadata$Latitude,
            lon = metadata$Longitude,
            alt = metadata$Altitude,
            sou = "PALAEO-RA",
            link = metadata$C3S.Link,
            units = units[var],
            stat = stats[var],
            metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),sep,switch(var1,"p"="PTC=N|PGC=Y","")),
            meta = meta_col,
            period = "month",
            time_offset = 0,
            keep_na = TRUE,
            outfile=paste0("PALAEO-RA_",metadata$Code,"_",startend,"_",var1,"_","monthly.tsv"))
  
}


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder),pattern="monthly", full.names=TRUE)) check_sef(f)

