#rm(list=ls())

library(dataresqc)
library(XLConnect)
library(readxl)
library(suncalc)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("../../../../Inventory_PALAEO-RA_digi.xls",sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)


############################
## subdaily
############################
# template <- readWorksheetFromFile("../../3_Revised/Marietta/USA_T3_US_Marietta_1788-1791_subdaily.xls",
#                                   sheet=1, startRow=7, header=T, colTypes="character")
### remove this later
template <- as.data.frame(read_excel("../../3_Revised/Marietta/USA_T3_US_Marietta_1788-1791_subdaily.xls",
                                  sheet=1, skip=6, col_names=T, col_types="text"))

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    dd = function(x) {return(22.5 * (match(toupper(x), directions) - 1))})

### time ###
time_offset <- as.numeric(lon)*12/180

Year <- as.integer(template[,"Year"])
Month <- as.integer(template[,"Month"])
Day <- as.integer(template[,"Day"])
dates <- as.Date(paste(Year,Month,Day, sep="-"))

sun <- suncalc::getSunlightTimes(dates[seq(1,length(dates),3)], lat=lat, lon=lon,keep=c("sunrise","sunset"))
ind <- seq(2,length(Year),3)
utc_2pm <- ISOdate(Year[ind],Month[ind],Day[ind],rep(c(14),length.out=length(ind)),rep(c(0),length.out=length(ind)),tz="") - time_offset*3600

Year <- matrix(unlist(t(data.frame(substr(sun[,4],1,4),substr(utc_2pm,1,4),substr(sun[,5],1,4)))),ncol=1,nrow=length(Year))
Month <- matrix(unlist(t(data.frame(substr(sun[,4],6,7),substr(utc_2pm,6,7),substr(sun[,5],6,7)))),ncol=1,nrow=length(Year))
Day <- matrix(unlist(t(data.frame(substr(sun[,4],9,10),substr(utc_2pm,9,10),substr(sun[,5],9,10)))),ncol=1,nrow=length(Year))
hours <- matrix(unlist(t(data.frame(substr(sun[,4],12,13),substr(utc_2pm,12,13),substr(sun[,5],12,13)))),ncol=1,nrow=length(Year))
minutes <- matrix(unlist(t(data.frame(substr(sun[,4],15,16),substr(utc_2pm,15,16),substr(sun[,5],15,16)))),ncol=1,nrow=length(Year))
orig_h <- rep(c("sunrise","2pm","sunset"), times=length(Year)/3)

dates_help <- ISOdate(Year,Month,Day, hours,minutes)

## check if dates correctly ordered - changes order in parent frame
check_chron_order(dates_help)

## check:
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)
startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],
                   ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

############## 
#### Temp F
############## 
dat <- template[,4]
unique(dat)
vardat <- conversions[["ta"]](dat)
  
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
  
meta_col <- paste0(meta_time, ifelse(dat=="NA",paste0("|orig=",dat),paste0("|orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_ta.tsv"))
  

############## 
####  Wind dir
############## 

convDIR <- c("southerly"="S","south"="S","Sy"="S","souh"="S","SWest"="SW","SEast"="SE","S to west"="SW",
             "Ny"="N","northerly"="N","north"="N","north wind"="N", "northerly wind"="N","NWest"="NW",
             "Nwest"="NW","NEast"="NE","West"="W","west"="W","east"="E","Wy"="W","Ey"="E","SW b W"="WSW",
             "SSWest"="SSW","Swest"="SW","westerly"="W", "S to west"="SW","SWest to W"="WSW","SW & west"="WSW",
             "SEast to S"="SSE")

directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")

dat <- template[,5]
unique(dat)

cuts <- c(" & fresh"," fresh")
dat_cut <- gsub(cuts[1],"",dat)
dat_cut <- gsub(cuts[2],"",dat_cut)

### change ditto to value registered the line before
dat_cut <- replace_dittos(dat_cut)
unique(dat_cut)

repl <- which(!dat_cut%in%directions)
dat_cut[repl] <- convDIR[match(dat_cut[repl], names(convDIR))]
names(dat_cut) <- NULL

vardat <- conversions[["dd"]](dat_cut)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time,"|orig=",dat)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_dd.tsv"))



## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)

