#rm(list=ls())

require(dataresqc)
require(XLConnect)
require(suncalc)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep("day",length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    piper <- rle(dat)
    pipes <- which(dat=="|")
    pipec <- cumsum(piper$lengths) + 1
    j <- which(piper$values=="|")
    period[pipec[j]] <- paste0(piper$lengths[j]+1, "day")     
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("../../../../Inventory_PALAEO-RA_digi.xls",sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
(lat <- as.numeric(metadata$Latitude))
(lon <- as.numeric(metadata$Longitude))
(alt <- as.numeric(metadata$Altitude))


conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    p  = function(x,z=NULL) round(dataresqc::convert_pressure(as.numeric(x),f = 25.4,lat = lat, alt = alt,atb=z), 1),
                    dd = function(x) {return(22.5 * (match(toupper(x), directions) - 1))},
                    rr = function(x) {round(as.numeric(x)*25.4,1)})


fil <- list.files("../../3_Revised/NewBedford/", full.names = T)
template <- readWorksheetFromFile(fil,sheet=1, startRow=8, colTypes="character")

time_offset <- as.numeric(lon)*12/180


############################
## daily
############################

Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- as.integer(template[,3])
dates <- as.Date(paste(Year,Month,Day, sep="-"))

hours <- rep(NA, length(Day))
minutes <- rep(NA, length(Day))

dates_help <- dates

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)

## check: 
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

############## 
#### Tx
############## 
dat <- template[,8]
vardat <- conversions[["ta"]](dat)
  
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
  
meta_col <- paste0(ifelse(is.na(vardat),paste0("orig=",dat),paste0("orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "Tx",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = "day",
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_Tx.tsv"))

############## 
#### Tn
############## 
dat <- template[,9]
vardat <- conversions[["ta"]](dat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(ifelse(is.na(vardat),paste0("orig=",dat),paste0("orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "Tn",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = "day",
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_Tn.tsv"))



############## 
####  Precip
############## 

col_rr <- 14
## check for pipe symbols and remove respc. value for the whole accuml.period
dat_list <- convert_pipes(template,col_rr)  # entfernt pipe-Daten
dat <- convert_NAs_for_rr(dat_list$dat)
vardat <- conversions[["rr"]](dat)

out <- data.frame(dat_list$Year,dat_list$Month,dat_list$Day,dat_list$hours,dat_list$minutes,vardat)

meta_col <- paste0(ifelse((vardat=="NA" | dat =="NA"),paste0("orig=",dat),paste0("orig=",dat,"in")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = dat_list$period,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_rr.tsv"))




######################################################## 
#################### SUBDAILY
####################################################### 

Year <- as.integer(rep(template[,1],each=4))
Month <- as.integer(rep(template[,2],each=4))
Day <- as.integer(rep(template[,3],each=4))
dates <- as.Date(paste(Year,Month,Day, sep="-"))

### check if these dates are really okay...
sun <- suncalc::getSunlightTimes(dates[seq(1,length(dates),4)], lat=lat, lon=lon,keep=c("sunrise","sunset"))
ind <- seq(2,length(dates),4)
utc_2pm <- ISOdate(Year[ind],Month[ind],Day[ind],rep(c(14),length.out=length(ind)),rep(c(0),length.out=length(ind)),tz="") - time_offset*3600
ind2 <- seq(4,length(dates),4)
utc_10pm <- ISOdate(Year[ind],Month[ind],Day[ind],rep(c(22),length.out=length(ind)),rep(c(0),length.out=length(ind)),tz="") - time_offset*3600

Year <- as.integer(as.vector(t(data.frame(substr(sun[,4],1,4),substr(utc_2pm,1,4),substr(sun[,5],1,4),substr(utc_10pm,1,4)))))
Month <- as.vector(t(data.frame(substr(sun[,4],6,7),substr(utc_2pm,6,7),substr(sun[,5],6,7),substr(utc_10pm,6,7))))
Day <- as.vector(t(data.frame(substr(sun[,4],9,10),substr(utc_2pm,9,10),substr(sun[,5],9,10),substr(utc_10pm,9,10))))
hours <- as.vector(t(data.frame(substr(sun[,4],12,13),substr(utc_2pm,12,13),substr(sun[,5],12,13),substr(utc_10pm,12,13))))
minutes <- as.vector(t(data.frame(substr(sun[,4],15,16),substr(utc_2pm,15,16),substr(sun[,5],15,16),substr(utc_10pm,15,16))))
orig_h <- rep(c("sunrise","2pm","sunset","10pm"), times=length(Year)/4)

dates_help <- ISOdate(Year,Month,Day,hours,minutes,tz="UTC")

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)

## check: 
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)
startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],
                   ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

############## 
#### ta
##############
dat <- as.vector(t(template[,4:7]))
vardat <- conversions[["ta"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_ta.tsv"))


############## 
#### Pressure
############## 
dat <- as.vector(t(template[,10:13]))
vardat <- conversions[["p"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"in")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = gsub("[[:space:]]","",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)), "|PTC=N|PGC=Y")),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_p.tsv"))




## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)

