#rm(list=ls())

require(dataresqc)
require(XLConnect)
require(suncalc)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, "1", sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("../../../../Inventory_PALAEO-RA_digi.xls",sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)
time_offset <- as.numeric(lon)*12/180

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    dd = function(x) {return(22.5 * (match(toupper(x), directions) - 1))},
                    wind_force = function(x) round(as.numeric(x),0),
                    p  = function(x,z=NULL) {
                      if(is.null(z)){
                        xx <- dataresqc::convert_pressure(as.numeric(x),f = 25.4,lat = lat, alt = alt)
                        } else{
                        xx <- ifelse(is.na(as.numeric(z)),dataresqc::convert_pressure(as.numeric(x),f = 25.4,lat = lat, alt = alt),
                                     dataresqc::convert_pressure(as.numeric(x),f = 25.4,lat = lat, alt = alt,atb=z))
                        }
                      return(xx)})

fil <- list.files("../../3_Revised/NewHaven/",pattern=metadata$Observer, full.names = T)
templates <- lapply(fil, function(f){readWorksheetFromFile(f,sheet=1, startRow=7, colTypes="character")})

################################
#### only for the Variables ta, dd, wind force
################################

## all columns y,m,d,h,ta,dir,wf
temp1 <- data.frame(templates[[1]][,c(1:7)])
temp2 <- data.frame(templates[[2]][,c(1:7)])
temp3 <- data.frame(templates[[3]][,c(1:7)])
temp4 <- data.frame(templates[[4]][,c(1:7)])

template <- abind::abind(temp1,temp2,temp3,temp4, along=1)

Year <- fill_val(as.integer(template[,1]))
Month <- fill_val(as.integer(template[,2]))
Day <- fill_val(as.integer(template[,3]))
dates <- as.Date(paste(Year,Month,Day, sep="-"))

## houres ohne sunset, sunrise, etc
hours_help <- substr(template[,4],12,13) 
minutes_help <- substr(template[,4],15,16)
## adjust exceptions to reasonable values
hours_help[which(template[,4]=="NA")] <- NA
minutes_help[which(template[,4]=="NA")] <- NA
hours_help[which(template[,4]=="PM")] <- "15"
minutes_help[which(template[,4]=="PM")] <- "00"
hours_help[which(template[,4]=="AM")] <- "10"
minutes_help[which(template[,4]=="AM")] <- "00"
hours_help[which(template[,4]==" 10 pm")] <- "22"
minutes_help[which(template[,4]==" 10 pm")] <- "00"
hours_help[which(template[,4]=="08:00.00")] <- "08"
minutes_help[which(template[,4]=="08:00.00")] <- "00"
hours_help[which(template[,4]=="21.00:00")] <- "21"
minutes_help[which(template[,4]=="21.00:00")] <- "00"
hours_help[which(template[,4]=="sunset 17:15:00")] <- "17"
minutes_help[which(template[,4]=="sunset 17:15:00")] <- "15"
hours_help[which(template[,4]=="noon")] <- "12"
minutes_help[which(template[,4]=="noon")] <- "00"
hours_help[which(template[,4]=="moon")] <- NA
minutes_help[which(template[,4]=="moon")] <- NA
hours_help[which(template[,4]=="night")] <- "22"
minutes_help[which(template[,4]=="night")] <- "00"
# midnight: for the first one, midnight is written for the next day!
midnight <- which(template[,4]=="midnight")
hours_help[midnight[2:length(midnight)]] <- "23"
minutes_help[midnight[2:length(midnight)]] <- "59"
hours_help[midnight[1]] <- "00"
minutes_help[midnight[1]] <- "00"
## there is additionally "moon", but not clear to me, what it stand for (it is in the morning)

## create fake hours for missing hours...
nahours <- which(is.na(hours_help) & !is.na(Day))
helpt <- unlist(sapply(rle(Day[nahours])$lengths, function(x) seq(6,16,length.out = x)) )
fakehours <- hours_help
fakehours[nahours] <- helpt
fakeminutes <- minutes_help
fakeminutes[nahours] <- rep("00",length(helpt))

## convert to utc
time_help <- ISOdate(Year,Month,Day,fakehours,fakeminutes) - time_offset*3600
Year <- as.integer(substr(time_help,1,4))
Month <- as.integer(substr(time_help,6,7))
Day <- as.integer(substr(time_help,9,10))
hours <- substr(time_help,12,13)
minutes <- substr(time_help,15,16)

## sunrise
sr_ind <- which(gsub("[[:space:]]","",template[,4])%in%c("sunrise"))
sunrise <- suncalc::getSunlightTimes(dates[sr_ind], lat=lat, lon=lon,keep=c("sunrise"))[,4]
Year[sr_ind] <- as.integer(substr(sunrise,1,4))
Month[sr_ind] <- as.integer(substr(sunrise,6,7))
Day[sr_ind] <- as.integer(substr(sunrise,9,10))
hours[sr_ind] <- substr(sunrise,12,13)
minutes[sr_ind] <- substr(sunrise,15,16)

# sunset
ss_ind <- which(gsub("[[:space:]]","",template[,4])%in%c("sunset"))
sunset <- suncalc::getSunlightTimes(dates[ss_ind], lat=lat, lon=lon,keep=c("sunset"))[,4]
Year[ss_ind] <- as.integer(substr(sunset,1,4))
Month[ss_ind] <- as.integer(substr(sunset,6,7))
Day[ss_ind] <- as.integer(substr(sunset,9,10))
hours[ss_ind] <- substr(sunset,12,13)
minutes[ss_ind] <- substr(sunset,15,16)

orig_h <- ifelse(nchar(template[,4])>14,substr(template[,4],12,19),template[,4])

dates_help <- ISOdate(Year,Month,Day,hours,minutes,tz="UTC")
hours[nahours] <- NA
minutes[nahours] <- NA

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)

## check: 
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],
                   ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

############## 
#### ta
##############
dat <- template[,5]
unique(dat)
# aa <- which(dat=="1900-03-04 00:00:00")
# template[(aa-20):(aa+10),]
vardat <- conversions[["ta"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_ta.tsv"))

############## 
#### dd
############## 
dat <- template[,6]
unique(dat)

## convert wind directions
convDIR <- c("E by S"="SE","W by N"="NW","W by S"="SW","S by W"="SW","SE by S"="SSE","S by W"="SW","NE by N"="NNE","N by E"="NE","N b. E"="NE",
             "N b E"="NE","NbE"="NE","WbN"="NW","SbE"="SE","SbW"="SW","n"="N","S b E"="SE", " NE"="NE","N all day"="N","S by E"="SE","E by N"="NE","N by W"="NW","ENN"="NNE","N byW"="NW","e"="E","S b W" ="SW")
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")

repl <- which(!dat%in%directions)
dat[repl] <- convDIR[match(dat[repl], names(convDIR))]
names(dat) <- NULL
unique(dat)

vardat <- conversions[["dd"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, paste0("|orig=",template[,6]))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_dd.tsv"))


################### 
#### wind force
################### 
dat <- template[,7]
unique(dat)
vardat <- conversions[["wind_force"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, paste0("|orig=",template[,7]))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "wind_force",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-5",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_wind_force.tsv"))




###################################################################################### 
#### barometer, only in templates 3 and 4, thus shorter period
#######################################################################################
temp_n3 <- data.frame(templates[[3]][,c(1:7,9)])
temp_n4 <- data.frame(templates[[4]][,c(1:8)])

template_bar <- abind::abind(temp_n3,temp_n4, along=1)

Year <- fill_val(as.integer(template_bar[,1]))
Month <- fill_val(as.integer(template_bar[,2]))
Day <- fill_val(as.integer(template_bar[,3]))
dates <- as.Date(paste(Year,Month,Day, sep="-"))

## houres ohne sunset, sunrise, etc
hours_help <- substr(template_bar[,4],12,13) 
minutes_help <- substr(template_bar[,4],15,16)
## adjust exceptions to reasonable values
hours_help[which(template_bar[,4]==" 10 pm")] <- "22"
minutes_help[which(template_bar[,4]==" 10 pm")] <- "00"
hours_help[which(template_bar[,4]=="night")] <- "22"
minutes_help[which(template_bar[,4]=="night")] <- "00"
hours_help[which(template_bar[,4]=="NA")] <- NA
minutes_help[which(template_bar[,4]=="NA")] <- NA


nahours <- which(is.na(hours_help) & !is.na(Day))
helpt <- unlist(sapply(rle(Day[nahours])$lengths, function(x) seq(6,16,length.out = x)) )
fakehours <- hours_help
fakehours[nahours] <- helpt
fakeminutes <- minutes_help
fakeminutes[nahours] <- rep("00",length(helpt))

## convert to utc
time_help <- ISOdate(Year,Month,Day,fakehours,fakeminutes) - time_offset*3600
Year <- as.integer(substr(time_help,1,4))
Month <- as.integer(substr(time_help,6,7))
Day <- as.integer(substr(time_help,9,10))
hours <- substr(time_help,12,13)
minutes <- substr(time_help,15,16)

## sunrise
sr_ind <- which(template_bar[,4]%in%c("sunrise"))
sunrise <- suncalc::getSunlightTimes(dates[sr_ind], lat=lat, lon=lon,keep=c("sunrise"))[,4]
Year[sr_ind] <- as.integer(substr(sunrise,1,4))
Month[sr_ind] <- as.integer(substr(sunrise,6,7))
Day[sr_ind] <- as.integer(substr(sunrise,9,10))
hours[sr_ind] <- substr(sunrise,12,13)
minutes[sr_ind] <- substr(sunrise,15,16)

## sunset
ss_ind <- which(gsub("[[:space:]]","",template_bar[,4])%in%c("sunset"))
sunset <- suncalc::getSunlightTimes(dates[ss_ind], lat=lat, lon=lon,keep=c("sunset"))[,4]
Year[ss_ind] <- as.integer(substr(sunset,1,4))
Month[ss_ind] <- as.integer(substr(sunset,6,7))
Day[ss_ind] <- as.integer(substr(sunset,9,10))
hours[ss_ind] <- substr(sunset,12,13)
minutes[ss_ind] <- substr(sunset,15,16)

orig_h <- ifelse(nchar(template_bar[,4])>14,substr(template_bar[,4],12,19),template_bar[,4])

dates_help <- ISOdate(Year,Month,Day,hours,minutes,tz="UTC")
hours[nahours] <- NA
minutes[nahours] <- NA

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
template <- template_bar
check_chron_order(dates_help)

## check: 
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],
                   ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

## barom
dat <- template[,8]
unique(dat)
vardat <- round(conversions[["p"]](dat),1)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(dat),paste0("|orig=",dat),paste0("|orig=",dat,"in")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),"|PTC=N|PGC=Y")),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_p.tsv"))



## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder),pattern="NewHaven_1", full.names=TRUE)) check_sef(f)

