## NOTES:
## - NewHaven_2 has quiet some NA-values in the time column in first template 1788-1791
## - it has to be reordered
## - last template (1813) does not contain time for morinng measurement->set to sunrise 
#rm(list=ls())

require(dataresqc)
require(XLConnect)
require(suncalc)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, "2", sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("../../../../Inventory_PALAEO-RA_digi.xls",sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]
lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)

time_offset <- as.numeric(lon)*12/180

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    dd = function(x) {return(22.5 * (match(toupper(x), directions) - 1))},
                    wind_force = function(x) round(as.numeric(x),0))


fil <- list.files("../../3_Revised/NewHaven/",pattern=metadata$Observer, full.names = T)
templates <- lapply(fil, function(f){readWorksheetFromFile(f,sheet=1, startRow=7, colTypes="character")})


############################
## TEMPLATE 3 by itself ->1800 ta and wind dir
############################

template <- data.frame(templates[[4]])

Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- fill_val(as.integer(c(template[,3])))
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- as.integer(substr(template[,4],12,13))
minutes <- as.integer(substr(template[,4],15,16))
orig_h <- c(substr(template[,4],12,22))

## reorder
## create dates_help to check order based on fake hours
dates_help <- ISOdate(Year,Month,Day,hours,minutes, tz="UTC")

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

############## 
#### ta
##############
dat <- template[,5]
vardat <- conversions[["ta"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_ta.tsv"))


############## 
#### dd
############## 
dat <- template[,6]
unique(dat)
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")

vardat <- conversions[["dd"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, paste0("|orig=",dat))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_dd.tsv"))


############################
## TEMPLATE 1 and 5 for tin and tout
############################

temp1 <- data.frame(templates[[1]][,c(1:5,7,6)], rep(NA,length(templates[[1]][,1])))
temp5 <- data.frame(templates[[5]][,c(1:4,8,5,7,6)])

template <- abind::abind(temp1,temp5, along=1)

Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- fill_val(template[,3])
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- as.integer(substr(template[,4],12,13))
minutes <- as.integer(substr(template[,4],15,16))

## 3. reorder 
nahours <- which(is.na(hours) & !is.na(Day))
helpt <- unlist(sapply(rle(Day[nahours])$lengths, function(x) seq(4,16,length.out = x)) )
fakehours <- hours
fakehours[nahours] <- helpt
fakeminutes <- minutes
fakeminutes[nahours] <- rep("00",length(helpt))

dates_help <- ISOdate(Year,Month,Day,fakehours,fakeminutes)

## set fakehours and minutes first!! to NA, then reorder - dpes not affect dates_help
hours[nahours] <- NA
minutes[nahours] <- NA

orig_h <- c(substr(template[,4],12,22))

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

############## 
#### tin
##############
dat <- gsub(" ([0-9]+) *","",template[,5]) ## remove stuff after space
unique(dat)
vardat <- conversions[["ta"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "atb",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_atb.tsv"))


############## 
#### ta
##############
dat <- template[,6]
unique(dat)
vardat <- conversions[["ta"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_ta.tsv"))


############################
## TEMPLATE 2,3,6 for thermometer...
############################

temp2 <- data.frame(templates[[2]][,c(1:7)])
temp3 <- data.frame(templates[[3]][,c(1:7)])
temp6 <- data.frame(templates[[6]][,c(1:7)])

template <- abind::abind(temp2,temp3,temp6, along=1)
ind6 <- (length(template[,1])-length(temp6[,1])+1):length(template[,1])
ind6_sun <- ind6[is.na(temp6[,4])]
ind6_2 <- ind6[!is.na(temp6[,4])]

Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- fill_val(as.integer(c(template[1:(ind6[1]-1),3], template[ind6,3])))
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- as.integer(substr(template[,4],12,13))
minutes <- as.integer(substr(template[,4],15,16))

## 1. adjust last template for sunrise?
sun_temp6<- suncalc::getSunlightTimes(dates[ind6_sun], lat=lat, lon=lon,keep=c("sunrise"))
hours[ind6_sun] <- substr(sun_temp6[,4],12,13)
minutes[ind6_sun] <- substr(sun_temp6[,4],15,16)

## 2. adjust last template for 12am and 1,2,10pm, adjust also day,mon,year because of utc shift
timeshift <- ifelse(template[ind6_2,4]=="12",0,12)
two_temp6 <- ISOdate(Year[ind6_2],Month[ind6_2],Day[ind6_2],template[ind6_2,4],rep("00",length(ind6_2)),tz="") + (timeshift*3600) - (time_offset*3600)
hours[ind6_2] <- substr(two_temp6,12,13)
Year[ind6_2] <- as.integer(substr(two_temp6,1,4))
Month[ind6_2] <- as.integer(substr(two_temp6,6,7))
Day[ind6_2] <- as.integer(substr(two_temp6,9,10))
minutes[ind6_2] <- substr(two_temp6,15,16)

## 3. convert other templates to utc, also y,m,d. 
nahours <- which(is.na(hours) & !is.na(Day))
helpt <- unlist(sapply(rle(Day[nahours])$lengths, function(x) seq(4,16,length.out = x)) )
fakehours <- hours
fakehours[nahours] <- helpt
fakeminutes <- minutes
fakeminutes[nahours] <- rep("00",length(helpt))

time_help <- ISOdate(Year[1:(ind6[1]-1)],Month[1:(ind6[1]-1)],Day[1:(ind6[1]-1)],fakehours[1:(ind6[1]-1)],fakeminutes[1:(ind6[1]-1)],tz="") - time_offset*3600
Year[1:(ind6[1]-1)] <- as.integer(substr(time_help,1,4))
Month[1:(ind6[1]-1)] <- as.integer(substr(time_help,6,7))
Day[1:(ind6[1]-1)] <- as.integer(substr(time_help,9,10))
hours[1:(ind6[1]-1)] <- substr(time_help,12,13)
minutes[1:(ind6[1]-1)] <- substr(time_help,15,16)

## 4. reorder
## create dates_help to check order based on fake hours
dates_help <- ISOdate(Year,Month,Day,hours,minutes, tz="UTC")

## set fakehours and minutes first!! to NA, then reorder - dpes not affect dates_help
hours[nahours] <- NA
minutes[nahours] <- NA
orig_h <- c(substr(template[1:(ind6[1]-1),4],12,22),template[ind6,4])

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

############## 
#### ta
##############
dat <- template[,5]
unique(dat)
vardat <- conversions[["ta"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_ta.tsv"))



############################
## TEMPLATE 1,2,3,6 for dd
############################

temp1 <- data.frame(templates[[1]][,c(1:7)])
temp2 <- data.frame(templates[[2]][,c(1:7)])
temp3 <- data.frame(templates[[3]][,c(1:7)])
temp6 <- data.frame(templates[[6]][,c(1:7)])

template <- abind::abind(temp1,temp2,temp3,temp6, along=1)
ind6 <- (length(template[,1])-length(temp6[,1])+1):length(template[,1])
ind6_sun <- ind6[is.na(temp6[,4])]
ind6_2 <- ind6[!is.na(temp6[,4])]

Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- fill_val(as.integer(c(template[1:(ind6[1]-1),3], template[ind6,3])))
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- as.integer(substr(template[,4],12,13))
minutes <- as.integer(substr(template[,4],15,16))

## 1. adjust last template for sunrise?
sun_temp6<- suncalc::getSunlightTimes(dates[ind6_sun], lat=lat, lon=lon,keep=c("sunrise"))
hours[ind6_sun] <- substr(sun_temp6[,4],12,13)
minutes[ind6_sun] <- substr(sun_temp6[,4],15,16)

## 2. adjust last template for 12am and 1,2,10pm, adjust also day,mon,year because of utc shift
timeshift <- ifelse(template[ind6_2,4]=="12",0,12)
two_temp6 <- ISOdate(Year[ind6_2],Month[ind6_2],Day[ind6_2],template[ind6_2,4],rep("00",length(ind6_2)),tz="") + (timeshift*3600) - (time_offset*3600)
hours[ind6_2] <- substr(two_temp6,12,13)
Year[ind6_2] <- as.integer(substr(two_temp6,1,4))
Month[ind6_2] <- as.integer(substr(two_temp6,6,7))
Day[ind6_2] <- as.integer(substr(two_temp6,9,10))
minutes[ind6_2] <- substr(two_temp6,15,16)

## 3. convert other templates to utc, also y,m,d. 
nahours <- which(is.na(hours) & !is.na(Day))
helpt <- unlist(sapply(rle(Day[nahours])$lengths, function(x) seq(4,16,length.out = x)) )
fakehours <- hours
fakehours[nahours] <- helpt
fakeminutes <- minutes
fakeminutes[nahours] <- rep("00",length(helpt))

time_help <- ISOdate(Year[1:(ind6[1]-1)],Month[1:(ind6[1]-1)],Day[1:(ind6[1]-1)],fakehours[1:(ind6[1]-1)],fakeminutes[1:(ind6[1]-1)],tz="") - time_offset*3600
Year[1:(ind6[1]-1)] <- as.integer(substr(time_help,1,4))
Month[1:(ind6[1]-1)] <- as.integer(substr(time_help,6,7))
Day[1:(ind6[1]-1)] <- as.integer(substr(time_help,9,10))
hours[1:(ind6[1]-1)] <- substr(time_help,12,13)
minutes[1:(ind6[1]-1)] <- substr(time_help,15,16)

## 4. reorder
## create dates_help to check order based on fake hours
dates_help <- ISOdate(Year,Month,Day,hours,minutes, tz="UTC")

## set fakehours and minutes first!! to NA, then reorder - dpes not affect dates_help
hours[nahours] <- NA
minutes[nahours] <- NA
orig_h <- c(substr(template[1:(ind6[1]-1),4],12,22),template[ind6,4])

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

############## 
#### dd
##############
dat <- template[,6]
unique(dat)
## convert wind directions
convDIR <- c("E by S"="SE","W by N"="NW","W by S"="SW","SE by S"="SSE","S by W"="SW","NE by N"="NNE",
             "N by E"="NE","S by E"="SE","E by N"="NE","N by W"="NW","ENN"="NNE","N byW"="NW", "SW to S"="SSW","S to SW"="SSW","WWW"="W","NbE"="NE","NbW"="NW", " NW"="NW",
             "SbE"="SE","WbN"="WN","NWbW"="WNW","NNW & NW"="NNW","N & NNW"="NNW", "N to NNE"="NNE","SEE"="ESE","NW "="NW", "WN"="NW")

directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")

dat <- replace_dittos(dat)
repl <- which(!dat%in%directions)
dat[repl] <- convDIR[match(dat[repl], names(convDIR))]
names(dat) <- NULL
unique(dat)

vardat <- conversions[["dd"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, paste0("|orig=",dat))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_dd.tsv"))


############################
## TEMPLATE 5 separately for dd
############################

template <- data.frame(templates[[5]])

Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- fill_val(as.integer(c(template[,3])))
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- as.integer(substr(template[,4],12,13))
minutes <- as.integer(substr(template[,4],15,16))

## 3. convert other templates to utc, also y,m,d. 
nahours <- which(is.na(hours) & !is.na(Day))
helpt <- unlist(sapply(rle(Day[nahours])$lengths, function(x) seq(4,16,length.out = x)) )
fakehours <- hours
fakehours[nahours] <- helpt
fakeminutes <- minutes
fakeminutes[nahours] <- rep("00",length(helpt))

## 4. reorder
## create dates_help to check order based on fake hours
dates_help <- ISOdate(Year,Month,Day,fakehours,fakeminutes)

## set fakehours and minutes first!! to NA, then reorder - dpes not affect dates_help
hours[nahours] <- NA
minutes[nahours] <- NA
orig_h <- c(substr(template[,4],12,22))

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

############## 
#### dd
##############
dat <- template[,6]
unique(dat)
## convert wind directions
convDIR <- c("E by S"="SE","W by N"="NW","W by S"="SW","SE by S"="SSE","S by W"="SW","NE by N"="NNE","W & WNW"="WNW","N & NNE"="NNE","E & ESE"="ESE",
             "N by E"="NE","S by E"="SE","E by N"="NE","N by W"="NW","ENN"="NNE","N byW"="NW", "SW to S"="SSW","S to SW"="SSW","WWW"="W","NbE"="NE","NbW"="NW", " NW"="NW",
             "SbE"="SE","WbN"="WN","NWbW"="WNW","NNW & NW"="NNW","N & NNW"="NNW", "N to NNE"="NNE","SEE"="ESE","NW "="NW", "WN"="NW")
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")

repl <- which(!dat%in%directions)
dat[repl] <- convDIR[match(dat[repl], names(convDIR))]
names(dat) <- NULL
unique(dat)

vardat <- conversions[["dd"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, paste0("|orig=",dat))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_dd.tsv"))


############## 
#### w
##############
dat <- template[,7]
unique(dat)
dat <- gsub(' *', '' , dat)
dat <- ifelse(is.na(as.numeric(dat)),as.numeric(substr(dat,1,1)),as.numeric(dat))

vardat <- conversions[["wind_force"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, paste0("|orig=",dat))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "wind_force",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-3",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_wind_force.tsv"))

##################
### Temp6 wind force only

template <- data.frame(templates[[6]][,c(1:7)])

ind6 <- 1:length(template[,1])
ind6_sun <- ind6[is.na(temp6[,4])]
ind6_2 <- ind6[!is.na(temp6[,4])]

Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- fill_val(as.integer(c(template[,3])))
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- as.integer(substr(template[,4],12,13))
minutes <- as.integer(substr(template[,4],15,16))

## 1. adjust last template for sunrise?
sun_temp6<- suncalc::getSunlightTimes(dates[ind6_sun], lat=lat, lon=lon,keep=c("sunrise"))
hours[ind6_sun] <- substr(sun_temp6[,4],12,13)
minutes[ind6_sun] <- substr(sun_temp6[,4],15,16)

## 2. adjust last template for 12am and 1,2,10pm, adjust also day,mon,year because of utc shift
timeshift <- ifelse(template[ind6_2,4]=="12",0,12)
two_temp6 <- ISOdate(Year[ind6_2],Month[ind6_2],Day[ind6_2],template[ind6_2,4],rep("00",length(ind6_2)),tz="") + (timeshift*3600) - (time_offset*3600)
hours[ind6_2] <- substr(two_temp6,12,13)
Year[ind6_2] <- as.integer(substr(two_temp6,1,4))
Month[ind6_2] <- as.integer(substr(two_temp6,6,7))
Day[ind6_2] <- as.integer(substr(two_temp6,9,10))
minutes[ind6_2] <- substr(two_temp6,15,16)

## 4. reorder
## create dates_help to check order based on fake hours
dates_help <- ISOdate(Year,Month,Day,hours,minutes, tz="UTC")
orig_h <- template[,4]

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)


############## 
#### w
##############
dat <- template[,7]
unique(dat)
dat <- gsub(' *', '' , dat)
dat <- ifelse(is.na(as.numeric(dat)),as.numeric(substr(dat,1,1)),as.numeric(dat))

vardat <- conversions[["wind_force"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, paste0("|orig=",dat))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "wind_force",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-3",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_wind_force.tsv"))


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)

