#rm(list=ls())

require(dataresqc)
require(XLConnect)
require(suncalc)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("../../../../Inventory_PALAEO-RA_digi.xls",sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == paste0(station_code,"_3") & inventory$Resolution == "subdaily"), ]
(lat <- as.numeric(metadata$Latitude))
(lon <- as.numeric(metadata$Longitude))
(alt <- as.numeric(metadata$Altitude))

time_offset <- as.numeric(lon)*12/180

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    rr = function(x) {round(as.numeric(x)*25.4,1)},
                    fs = function(x) {round(as.numeric(x)*25.4,1)},
                    p  = function(x,z=NULL) {
                      if(is.null(z)){xx <- convert_pressure(as.numeric(x),f = 25.4,lat = lat, alt = alt)} else{
                        xx <- ifelse(is.na(as.numeric(z)),convert_pressure(as.numeric(x),f = 25.4,lat = lat, alt = alt),convert_pressure(as.numeric(x),f = 25.4,lat = lat, alt = alt,atb=z))}
                      return(round(xx,2))}
                    )


fil <- list.files("../../3_Revised/NewHaven/",pattern="1811-1821", full.names = T)
template <- lapply(fil, function(f){readWorksheetFromFile(f,sheet=1, startRow=8, colTypes="character")})[[1]]


############################
## subdaily ta and p
############################

Year <- rep(as.integer(template[,1]),each=3)
Month <- rep(as.integer(template[,2]),each=3)
Day <- rep(as.integer(template[,3]),each=3)
dates <- as.Date(paste(Year,Month,Day, sep="-"))

## get sunrise and convert all to utc
hours <- rep(NA, length(Day))
minutes <- rep(NA, length(Day))
ind_time <- seq(1,length(Year),3)

sunrise <- suncalc::getSunlightTimes(dates[seq(1,length(dates),3)], lat=lat, lon=lon,keep=c("sunrise"))[,4]
hours[ind_time] <- substr(sunrise,12,13)
minutes[ind_time] <- substr(sunrise,15,16)
## no changes in day, because this is only +around 4 hours

time_help <- ISOdate(Year[-ind_time],Month[-ind_time],Day[-ind_time],rep(c("13","22"),times=length(template[,1])),rep(c("30","00"),times=length(template[,1]))) - time_offset*3600

Year[-ind_time] <- as.integer(substr(time_help,1,4))
Month[-ind_time] <- as.integer(substr(time_help,6,7))
Day[-ind_time] <- as.integer(substr(time_help,9,10))
hours[-ind_time] <- as.integer(substr(time_help,12,13))
minutes[-ind_time] <- as.integer(substr(time_help,15,16))

orig_h <- rep(c("SR","1/2pm","10pm"), length.out=length(Year))

dates_help <- ISOdate(Year,Month,Day,hours,minutes)

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)

## check: 
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))


meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

############## 
#### ta
##############
dat <- as.vector(t(template[,4:6]))
vardat <- conversions[["ta"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"F")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_ta.tsv"))



############## 
#### p
##############
dat <- as.vector(t(template[,8:10]))
unique(dat)
vardat <- conversions[["p"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"in")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)),"|PTC=N|PGC=Y")),
          meta = meta_col,
          period = 0,
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_p.tsv"))


#####################################################################
#### Daily  ########################################################
#####################################################################


fil <- list.files("../../3_Revised/NewHaven/",pattern="1811-1821", full.names = T)
template <- lapply(fil, function(f){readWorksheetFromFile(f,sheet=1, startRow=8, colTypes="character")})[[1]]

Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- as.integer(template[,3])
dates <- as.Date(paste(Year,Month,Day, sep="-"))
hours <- rep(NA, length(Day))
minutes <- rep(NA, length(Day))

dates_help <- ISOdate(Year,Month,Day,rep("13",times=length(Year)),rep("00",times=length(Year)))

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)
## check: 
any(order(dates_help)[2:(length(Year))]-order(dates_help)[1:(length(Year)-1)]!=1)

startend <- paste0(Year[1],ClimIndVis:::pad2(Month[1],2),ClimIndVis:::pad2(Day[1],2),"-",Year[length(Year)],ClimIndVis:::pad2(Month[length(Month)],2),ClimIndVis:::pad2(Day[length(Day)],2))

#meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)

############## 
#### rr
##############
col_rr <- 11
## check for pipe symbols and remove respc. value for the whole accuml.period
dat_list <- convert_pipes(template,col_rr=col_rr, dm=1,per="day")  # entfernt pipe-Daten
dat <- convert_NAs_for_rr(dat_list$dat)

vardat <- conversions[["rr"]](dat)
unique(vardat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(ifelse(is.na(vardat),paste0("orig=",dat),paste0("orig=",dat,"in")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = "day",
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_rr.tsv"))


############## 
#### fs
##############
col_rr <- 12
## check for pipe symbols and remove respc. value for the whole accuml.period
dat_list <- convert_pipes(template,col_rr=col_rr, dm=1,per="day")  # entfernt pipe-Daten
dat <- convert_NAs_for_rr(dat_list$dat)
unique(dat)
vardat <- conversions[["rr"]](dat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(ifelse(is.na(vardat),paste0("orig=",dat),paste0("orig=",dat,"in")))
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "fs",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = "day",
          time_offset = 0,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",startend,"_fs.tsv"))



## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder),pattern="_3", full.names=TRUE)) check_sef(f)

