## Note: Etat du Ciel not formatted 
## Finished by Yuri

#rm(list=ls())
#setwd("/scratch3/PALAEO-RA/DataRescue/Projects/USA/Scripts/NewYork")

require(dataresqc)
require(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("../../../../Inventory_PALAEO-RA_digi.xls",sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]

lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
alt <- as.numeric(metadata$Altitude)

############################
## subdaily
############################
template <- readWorksheetFromFile("../../3_Revised/NewYork/USA_T3_US_NewYork_1782-1784_subdaily.xls",
                                  sheet=1, startRow=8, header=FALSE, colTypes="character")
template[,1] <- fill_val(template[,1])
template[,2] <- fill_val(template[,2])
template[,3] <- fill_val(template[,3])

conversions <- list(ta = function(x) round(as.numeric(x) * 1.25, 1),
                    p  = function(x) dataresqc::convert_pressure(as.numeric(x),f = 27.07,lat = lat, alt = alt),
                    dd = function(x) 22.5 * (match(toupper(x), directions) - 1),
                    rr = function(x) round(as.numeric(x)*25.4,1))

### time ###
time_offset <- as.numeric(lon)*12/180
Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- as.integer(template[,3])
dates <- as.Date(paste(Year,Month,Day, sep="-"))
time <- ifelse(nchar(template[,4])>10, substr(template[,4],12,16), template[,4])
hours <- as.integer(substr(time,1,2))
minutes <- as.integer(substr(time,4,5))
orig_h <- time

############## 
#### Temp Réaumur
##############
j <- grep("-", template[,6])
template[j,5] <- paste0("-", template[j,5])
dat <- template[,5]
vardat <- conversions[["ta"]](dat)
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
out$hours <- as.character(out$hours)
out$minutes <- as.character(out$minutes)
meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)
meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"R")))

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


############## 
#### Pressure
############## 
pressure1 <- as.numeric(template[,7])
pressure2 <- as.numeric(template[,8])
dat <- pressure1 + pressure2/12
vardat <- round(conversions[["p"]](dat), 1)
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
meta_col <- paste0(meta_time, "|orig=", template[,7], ".", template[,8], "Pin")
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|PTC=N|PGC=Y"),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


############## 
####  Wind dir
############## 
dat <- template[,9]
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSO", "SO", "OSO", "O", "ONO", "NO", "NNO")
vardat <- conversions[["dd"]](dat)
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
meta_col <- paste0(meta_time,"|orig=",dat)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
