##

#rm(list=ls())
#setwd("/scratch3/PALAEO-RA/DataRescue/Projects/USA/Scripts/Newport")

require(dataresqc)
require(XLConnect)
require(suncalc)
require(lubridate)

#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("../../../../Inventory_PALAEO-RA_digi.xls",sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]

lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
##### Altitude missing
alt <- as.numeric(metadata$Altitude)


############################
## subdaily
############################
all_temp <- list.files("../../3_Revised/Newport/", full.names = T)

templates <- lapply(all_temp[1:4], 
                    function(fi){readWorksheetFromFile(fi,sheet=1, startRow=7, colTypes="character")})


templates <- lapply(templates, function(x) {
  colnames(x) <- NULL
  return(x)
})

## restructure first file
yr1 <- templates[[1]][,1]
mo1 <- templates[[1]][,2]
day1 <- templates[[1]][,3]
h1 <- templates[[1]][,4]
site1 <- templates[[1]][,5]
thermo1 <- templates[[1]][,6]
baro1 <- rep(NA, length(h1))
wigterm1 <- rep(NA, length(h1))
windir1 <- templates[[1]][,7]
winfor1 <- rep(NA, length(h1))
notes1 <- templates[[1]][,8]
temp1 <- as.matrix(data.frame(yr1,mo1,day1,h1,site1,thermo1,baro1,wigterm1,windir1,winfor1,notes1))

## restructure third file
yr3 <- templates[[3]][,1]
mo3 <- templates[[3]][,2]
day3 <- templates[[3]][,3]
h3 <- templates[[3]][,4]
site3 <- templates[[3]][,5]
thermo3 <- templates[[3]][,6]
baro3 <- rep(NA, length(h3))
wigterm3 <- rep(NA, length(h3))
windir3 <- templates[[3]][,7]
winfor3 <- templates[[3]][,8]
notes3 <- templates[[3]][,9]
temp3 <- as.matrix(data.frame(yr3,mo3,day3,h3,site3,thermo3,baro3,wigterm3,windir3,winfor3,notes3))

## restructure fourth file
yr4 <- templates[[4]][,1]
mo4 <- templates[[4]][,2]
day4 <- templates[[4]][,3]
h4 <- templates[[4]][,4]
site4 <- rep(NA,length(h4))
thermo4 <- templates[[4]][,5]
baro4 <- rep(NA, length(h4))
wigterm4 <- rep(NA, length(h4))
windir4 <- templates[[4]][,6]
winfor4 <- templates[[4]][,7]
notes4 <- templates[[4]][,8]
temp4 <- as.matrix(data.frame(yr4,mo4,day4,h4,site4,thermo4,baro4,wigterm4,windir4,winfor4,notes4))

template <- abind::abind(temp1, templates[[2]], temp3, temp4, along=1)

template[,1] <- fill_val(template[,1])
template[,2] <- fill_val(template[,2])
template[,3] <- fill_val(template[,3])
template[,4] <- fill_val(template[,4])

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    p  = function(x) convert_pressure(as.numeric(x),f=25.4,lat=lat,alt=alt),
                    dd = function(x) round(11.25 * (match(x, directions) - 1), 0))

### time ###
time_offset <- as.numeric(lon)*12/180
Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- as.integer(template[,3])
dates <- as.Date(paste(Year, Month, Day, sep="-"))
hours <- as.integer(substr(template[,4],12,13))
minutes <- as.integer(substr(template[,4],15,16))
orig_h <- substr(template[,4], 12, 16)
orig_h[!grepl("[:]",template[,4])] <- template[!grepl("[:]",template[,4]), 4]

## sunrise/sunset
sunrise_times <- suncalc::getSunlightTimes(dates, as.numeric(metadata$Latitude), as.numeric(metadata$Longitude), 
                                           keep="sunrise", tz="GMT")$sunrise + time_offset * 3600
sunset_times <- suncalc::getSunlightTimes(dates, as.numeric(metadata$Latitude), as.numeric(metadata$Longitude), 
                                          keep="sunset", tz="GMT")$sunset + time_offset * 3600
j <- which(template[,4]=="sunrise")
hours[j] <- hour(sunrise_times[j])
minutes[j] <- minute(sunrise_times[j])
j <- which(template[,4]=="sunset")
hours[j] <- hour(sunset_times[j])
minutes[j] <- minute(sunset_times[j])


############## 
#### Temp F
##############
dat <- template[,6]
vardat <- conversions[["ta"]](dat)
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
neworder <- order(out$Year,out$Month,out$Day,out$hours,out$minutes)
out <- out[neworder, ]
meta_time <- paste0("orig.date=", dates, "|orig.time=", orig_h)
meta_col <- paste0(meta_time, "|orig=", dat, "F|site=", template[,5])
meta_col <- meta_col[neworder]
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)

############## 
####  Wind dir
############## 
dat <- template[,9]
dat <- gsub("by", "b", dat)
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", "EbS", "ESE", "SEbE",
                "SE", "SEbS", "SSE", "SbE", "S", "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS",
                "W", "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")
vardat <- conversions[["dd"]](dat)
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
out <- out[neworder, ]
meta_col <- paste0(meta_time, "|orig=", template[,9])
meta_col <- meta_col[neworder]
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)

############## 
####  Wind force
############## 
dat <- template[,10]
vardat <- as.character(as.numeric(dat))
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
out <- out[neworder, ]
meta_col <- paste0(meta_time,"|orig=",dat)
meta_col <- meta_col[neworder]
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-6",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE)

############## 
#### Pressure
############## 
dat <- as.numeric(template[,7])
vardat <- round(conversions[["p"]](dat), 1)
out <- data.frame(Year,Month,Day,hours,minutes,vardat)
out <- out[neworder, ]
meta_col <- paste0(meta_time, "|orig=", template[,7], "in")
meta_col <- meta_col[neworder]
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|PTC=N|PGC=Y"),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = FALSE)


## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)
