#rm(list=ls())
#setwd("/scratch3/PALAEO-RA/DataRescue/Projects/USA/Scripts/Waterton")

require(dataresqc)
require(XLConnect)


#### functions for sef files...
fill_val <- function(x){
  
  ind <- which(!is.na(x))
  #ind <- switch((ind[length(ind)]==length(x))+1, c(ind),ind)
  for(i in 1:(length(ind))){
    if(i==length(ind)){
      x[ind[i]:length(x)] <- x[ind[i]]
    } else {
      x[ind[i]:(ind[i+1]-1)] <- x[ind[i]]
    }
  }
  return(x)
}


check_chron_order <- function(dates_help){
  
  env <- parent.frame()
  
  new_order <- order(dates_help)
  
  if(any((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)){
    ## order dates
    #aa <- which((new_order[2:(length(new_order))] - new_order[1:(length(new_order)-1)])!=1)
    
    message("Dates are reordered. Template as well!")
    
    env$Year <- as.numeric(Year[new_order])
    env$Month <- as.numeric(Month[new_order])
    env$Day <- as.numeric(Day[new_order])
    env$hours <- as.numeric(hours[new_order])
    env$minutes <- as.numeric(minutes[new_order])
    env$dates <- dates[new_order]
    env$orig_h <- orig_h[new_order]
    env$dates_help <- dates_help[new_order]
    env$template <- template[new_order,]
  }
}


replace_dittos <- function(dat_cut){
  
  dittos <- which(dat_cut%in%c("ditto","ditte"))
  
  for(d in dittos){
    
    dat_before <- dat_cut[1:(d-1)]
    dat_cut[d] <- tail(dat_before[!is.na(dat_before)],1)
    
  }
  
  return(dat_cut)
  
}


convert_NAs_for_rr <- function(x){
  
  realna <- which(x=="NA") 
  x[is.na(x)] <- 0
  x[realna] <- "NA"
  return(x)
  
}


convert_pipes <- function(x, col_rr,dm,per="p1day"){
  
  env <- parent.frame()
  dat <- x[,col_rr]
  period <- rep(per,length(dat))
  
  if(any(dat[!is.na(dat)]=="|")){
    
    pipes <- which(dat=="|")
    pipel <- rle(diff(pipes))$length
    
    if(any(pipel>dm)){
      wpipe <- pipes[which(pipel>dm)+1]
      period[wpipe] <- paste0(pipel[pipel>dm],"day" )
      
    }
    period <- period[-pipes]
    dat_list <- list(Year=env$Year[-pipes],
                     Month=env$Month[-pipes],
                     Day=env$Day[-pipes],
                     hours=env$hours[-pipes],
                     minutes=env$minutes[-pipes],
                     dat=dat[-pipes], 
                     period=period,
                     nopipes=c(1:length(env$Year))[-pipes])
    
  } else {
    message("no pipes found")
    
    # this should pick up from environment
    dat_list <- list(Year=env$Year,
                     Month=env$Month,
                     Day=env$Day,
                     hours=env$hours,
                     minutes=env$minutes,
                     dat=x[,col_rr],
                     period=period,
                     nopipes=1:length(env$Year))
  }
  
  return(dat_list)
  
}


## Create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

## Create output directory (if necessary)
if (!name_folder %in% list.dirs("../../4_Formatted/",full.names=FALSE)) {
  dir.create(paste0("../../4_Formatted/", name_folder))
}

## Read inventory + meta
inventory <- readWorksheetFromFile("../../../../Inventory_PALAEO-RA_digi.xls",sheet=1, colTypes="character")
metadata <- inventory[which(inventory$Code == station_code & inventory$Resolution == "subdaily"), ]

lat <- as.numeric(metadata$Latitude)
lon <- as.numeric(metadata$Longitude)
##### Altitude fehlt
alt <- as.numeric(metadata$Altitude)


############################
## subdaily
############################
template <- readWorksheetFromFile("../../3_Revised/Waterton/USA_T3_US_Waterton_1775_subdaily.xls",
                                  sheet=1, startRow=7, header=TRUE, colTypes="character")

## combining time columns
time1 <- as.numeric(template[,4])
time2 <- template[,5]
time <- ifelse(time2=="e",time1+12,ifelse(time2=="m",time1,"NA"))
time <- as.numeric(time)

template[,4] <- time
template <- template[,-5]

conversions <- list(ta = function(x) round((as.numeric(x) - 32) * 5 / 9, 1),
                    p  = function(x,z) {
                      xx <- ifelse(is.na(as.numeric(z)),dataresqc::convert_pressure(as.numeric(x),f = 25.4,lat = lat, alt = alt),
                                   dataresqc::convert_pressure(as.numeric(x),f = 25.4,lat = lat, alt = alt,atb=z))
                      return(xx)},
                    dd = function(x) {return(22.5 * (match(toupper(x), directions) - 1))},
                    rr = function(x) {round(as.numeric(x)*25.4,1)})

### time ###
time_offset <- as.numeric(lon)*12/180

Year <- as.integer(template[,1])
Month <- as.integer(template[,2])
Day <- as.integer(template[,3])
dates <- as.Date(paste(Year,Month,Day, sep="-"))

hours <- time-(time-floor(time))
minutes <- time-floor(time)
minutes <- ifelse(minutes==0.25,15,ifelse(minutes==0.5,30,ifelse(minutes==0.75,45,0)))

template2 <- readWorksheetFromFile("../../3_Revised/Waterton/USA_T3_US_Waterton_1775_subdaily.xls",
                                  sheet=1, startRow=7, header=TRUE, colTypes="character")

orig_h <- paste0(template2[,4],template2[,5])

dates_help <- as.Date(paste0(dates, hours,":",minutes))

## check if dates correctly ordered - changes order in parent frame
#  note that the data needs to be called "template" to sort it correctly!
check_chron_order(dates_help)


############## 
#### Temp F
##############
message("ta")
dat <- template[,7]
vardat <- conversions[["ta"]](dat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_time <- paste0("orig.date=",dates,"|orig.time=",orig_h)
meta_col <- paste0(meta_time, ifelse(is.na(vardat),paste0("|orig=",dat),paste0("|orig=",dat,"F")))

write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",metadata$First.Year,"-",metadata$Last.Year,"_ta.tsv"))

############## 
####  Wind dir
############## 

dat <- template[,8]
unique(dat)


convDIR <- c("E by S"="SE","W by N"="NW","W by S"="SW","SE by S"="SSE","S by W"="SW","NE by N"="NNE",
             "N by E"="NE","E by N"="NE","N by W"="NW")
directions <- c("N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW")

repl <- which(!dat%in%directions)
dat[repl] <- convDIR[match(dat[repl], names(convDIR))]
names(dat) <- NULL
unique(dat)

vardat <- conversions[["dd"]](dat)

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time,"|orig=",dat)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",metadata$First.Year,"-",metadata$Last.Year,"_dd.tsv"))

############## 
####  Wind force
############## 

dat <- template[,9]
unique(dat)

vardat <- as.character(as.numeric(dat))

out <- data.frame(Year,Month,Day,hours,minutes,vardat)

meta_col <- paste0(meta_time,"|orig=",dat)
write_sef(out,
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "wind_force",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "1-4",
          stat = "point",
          metaHead = gsub("[[:space:]]", "",paste0(ifelse(is.na(metadata$Observer), "", paste0("Observer=", metadata$Observer)))),
          meta = meta_col,
          period = 0,
          time_offset = time_offset,
          keep_na = TRUE,
          outfile = paste0("PALAEO-RA_",metadata$Code,"_",metadata$First.Year,"-",metadata$Last.Year,"_w.tsv"))

## Check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) check_sef(f)

